import random
from raga import *
import pandas as pd
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def img_url(x):
    file = x.split("/")[-1]
    return StringElement(f"https://raga-testing-storage.s3.ap-south-1.amazonaws.com/1/selected_coco_images/{file}")

def annotation_v1(row):
    AnnotationsV1 = ImageDetectionObject()
    for detection in row["AnnotationsV1"]:
        AnnotationsV1.add(ObjectDetection(Id=detection['Id'], ClassId=detection['ClassId'], ClassName=detection['ClassName'], Confidence=detection['Confidence'], BBox= detection['BBox'], Format="xywh_normalized"))
    return AnnotationsV1

def model_inferences(row):
    ModelInferences = ImageDetectionObject()
    for detection in row["ModelInferences"]:
        ModelInferences.add(ObjectDetection(Id=detection['Id'], ClassId=detection['ClassId'], ClassName=detection['ClassName'], Confidence=detection['Confidence'], BBox= detection['BBox'], Format="xywh_normalized"))
    return ModelInferences

def imag_vectors_m1(row):
    ImageVectorsM1 = ImageEmbedding()
    for embedding in row["ImageVectorsM1"]:
        ImageVectorsM1.add(Embedding(embedding))
    return ImageVectorsM1

def roi_vectors_m1(row):
    ROIVectorsM1 = ROIEmbedding()
    for embedding in row["ROIVectorsM1"]:
        ROIVectorsM1.add(id=embedding.get("Id"), embedding_values=embedding.get("embedding"))
    return ROIVectorsM1

def generate_random():
    classes = [
        "Yes",
        "No"
    ]
    return random.choice(classes)

def json_parser(json_file):
    df = pd.read_json(json_file)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df["ImageId"].apply(lambda x: StringElement(x))
    data_frame["ImageUri"] = df["SourceLink"].apply(lambda x: img_url(x))
    data_frame["SourceLink"] = df["SourceLink"].apply(lambda x: StringElement(x))
    data_frame["TimeOfCapture"] = df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["Reflection"] = df.apply(lambda row: StringElement(random.choice(["Yes", "No"])), axis=1)
    data_frame["Overlap"] = df.apply(lambda row: StringElement(random.choice(["Yes", "No"])), axis=1)
    data_frame["CameraAngle"] = df.apply(lambda row: StringElement(random.choice(["Yes", "No"])), axis=1)
    data_frame["AnnotationsV1"] = df.apply(annotation_v1, axis=1)
    data_frame["ModelInferences"] = df.apply(model_inferences, axis=1)
    data_frame["ImageVectorsM1"] = df.apply(imag_vectors_m1, axis=1)
    data_frame["ROIVectorsM1"] = df.apply(roi_vectors_m1, axis=1)
    return data_frame

pd_data_frame = json_parser("./assets/COCO_engg_final_updated.json")

#### Want to see in csv file uncomment line below ####
# data_frame_extractor(json_parser("./assets/COCO_engg_final_updated.json")).to_csv("./assets/COCO_engg_final_updated_100.csv", index=False)

schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement())
schema.add("SourceLink", FeatureSchemaElement())
schema.add("Reflection", AttributeSchemaElement())
schema.add("CameraAngle", AttributeSchemaElement())
schema.add("Overlap", AttributeSchemaElement())
schema.add("AnnotationsV1", InferenceSchemaElement(model="GT"))
schema.add("ModelInferences", InferenceSchemaElement(model="ModelB"))
schema.add("ROIVectorsM1", RoiEmbeddingSchemaElement(model="ROIModel"))
schema.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="imageModel"))

run_name = f"run-failure-mode-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="dE4Zs3iT8LnYPP3KNNsq", secret_key="JzDgIoB1irU9V2vbHtvTSNLvRzAzjbJZab2cZLos", host="http://13.126.220.245:8080")

cred = DatasetCreds(arn="arn:aws:iam::527593518644:role/s3-role")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="fma_image_object_detection_ds-v6", data=pd_data_frame, schema=schema, creds=cred)

#load schema and pandas data frame
test_ds.load()


from abc import ABC
import json
from typing import Any, Dict, List, Optional
from datetime import datetime
import pandas as pd

class RagaSchemaElement(ABC):
    def __init__(self):
        self.type = ""
        self.model = ""
        self.ref_col_name = ""
        self.label_mapping = {}
        self.schema = ""

class PredictionSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "imageName"

class ImageUriSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "imageUri"

class MaskUriSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "maskUri"

class TimeOfCaptureSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "timestamp"

class FeatureSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "feature"
class AttributeSchemaElement(RagaSchemaElement):
    def __init__(self):
        super().__init__()
        self.type = "attribute"

class InferenceSchemaElement(RagaSchemaElement):
    def __init__(self, model:str):
        super().__init__()
        from raga.constants import REQUIRED_ARG_V2

        if not isinstance(model, str) or not model: 
            raise ValueError(f"{REQUIRED_ARG_V2.format('model', 'str')}")
        self.type = "inference"
        self.model = model

class VideoInferenceSchemaElement(RagaSchemaElement):
    def __init__(self, model:str):
        super().__init__()
        from raga.constants import REQUIRED_ARG_V2

        if not isinstance(model, str) or not model: 
            raise ValueError(f"{REQUIRED_ARG_V2.format('model', 'str')}")
        self.type = "videoInference"
        self.model = model

class ImageEmbeddingSchemaElement(RagaSchemaElement):
    def __init__(self, model:str="", ref_col_name:str=""):
        super().__init__()
        self.type = "imageEmbedding"
        self.model = model
        self.ref_col_name = ref_col_name


class ImageClassificationSchemaElement(RagaSchemaElement):
    def __init__(self, model:str, ref_col_name:str=""):
        super().__init__()
        from raga.constants import REQUIRED_ARG_V2
        if not isinstance(model, str) or not model: 
            raise ValueError(f"{REQUIRED_ARG_V2.format('model', 'str')}")
        self.type = "classification"
        self.model = model
        self.ref_col_name = ref_col_name

class TIFFSchemaElement(RagaSchemaElement):
     def __init__(self, label_mapping:dict, schema:str="", model:str=""):
        super().__init__()
        from raga.constants import REQUIRED_ARG_V2
        if not isinstance(label_mapping, dict) or not label_mapping: 
            raise ValueError(f"{REQUIRED_ARG_V2.format('model', 'str')}")
        # Check that the label_mapping keys are integers and values are strings
        for key, value in label_mapping.items():
            if not isinstance(key, int) or not isinstance(value, str):
                raise ValueError(f"{REQUIRED_ARG_V2.format('label_mapping', 'str')}")
        self.type = "blob"
        self.label_mapping = label_mapping
        self.schema = schema
        self.model = model

class SemanticSegmentationSchemaElement(RagaSchemaElement):
     def __init__(self):
        super().__init__()
        self.type = "imageSegmentation"

class RoiEmbeddingSchemaElement(RagaSchemaElement):
    def __init__(self, model:str, ref_col_name:str=""):
        super().__init__()
        from raga.constants import REQUIRED_ARG_V2
        if not isinstance(model, str) or not model: 
            raise ValueError(f"{REQUIRED_ARG_V2.format('model', 'str')}")
        self.type = "roiEmbedding"
        self.model = model
        self.ref_col_name = ref_col_name

class MistakeScoreSchemaElement(RagaSchemaElement):
    def __init__(self, ref_col_name:str=""):
        super().__init__()
        self.type = "mistakeScores"
        self.ref_col_name = ref_col_name



class RagaSchema():
    def __init__(self):
        self.columns = list()

    def validation(self, column_name: str, ragaSchemaElement:RagaSchemaElement):
        from raga import REQUIRED_ARG_V2
        if not isinstance(column_name, str) or not column_name: 
            raise ValueError(f"{REQUIRED_ARG_V2.format('column_name', 'str')}")
        if not isinstance(ragaSchemaElement, RagaSchemaElement): 
            raise ValueError(f"{REQUIRED_ARG_V2.format('ragaSchemaElement', 'instance of the RagaSchemaElement')}")
        return True
     
    def add(self, column_name: str, ragaSchemaElement:RagaSchemaElement):
        self.validation(column_name, ragaSchemaElement)
        self.columns.append({"customerColumnName":column_name, "type":ragaSchemaElement.type, "modelName":ragaSchemaElement.model, "refColName":ragaSchemaElement.ref_col_name, "columnArgs":{"labelMapping":ragaSchemaElement.label_mapping, "schema":ragaSchemaElement.schema}})

class StringElement():
    def __init__(self, value:str):
        self.value = value

    def get(self):
        return self.value
    
class FloatElement():
    def __init__(self, value:float):
        self.value = value

    def get(self):
        return self.value
    
class TimeStampElement():
    def __init__(self, date_time:datetime):
        self.date_time = date_time

    def get(self):
        return self.date_time
    
class AggregationLevelElement():
    def __init__(self):
        self.levels = []

    def add(self, level:str):
        assert isinstance(level, str) and level, "level is required and must be str."
        self.levels.append(level)

    def get(self):
        return self.levels
    
class ModelABTestTypeElement():
    def __init__(self, type:str):
        self.type = type
        if self.type not in ["labelled", "unlabelled"]:
            raise ValueError("Invalid value for 'type'. Must be one of: ['labelled', 'unlabelled'].")

    def get(self):
        return self.type    

       
class ModelABTestRules():
    def __init__(self):
        self.rules = []

    def add(self, metric:str, IoU:float, _class:str, threshold:float):
        from raga import REQUIRED_ARG_V2

        assert isinstance(metric, str) and metric, f"{REQUIRED_ARG_V2.format('metric', 'str')}"
        assert isinstance(_class, str) and _class,f"{REQUIRED_ARG_V2.format('_class', 'str')}"
        assert isinstance(IoU, float) and IoU, f"{REQUIRED_ARG_V2.format('IoU', 'float')}"
        assert isinstance(threshold, float) and threshold, f"{REQUIRED_ARG_V2.format('threshold', 'float')}"
        self.rules.append({ "metric" : metric, "iou": IoU,  "class": _class, "threshold":threshold })

    def get(self):
        return self.rules
    
class FMARules():
    def __init__(self):
        self.rules = []

    def add(self, 
            metric:str, 
            metric_threshold:Optional[float]=None, 
            label:Optional[str]=None, 
            conf_threshold:Optional[float]=None, 
            iou_threshold:Optional[float]=None, 
            weights:Optional[dict]=None,
            type:Optional[str]=None,
            background_label:Optional[str]=None,
            include_background:Optional[bool]=False
            ) -> None:
        """
        Add a rule to the FMA rules list.

        Args:
            metric (str): The metric name.
            metric_threshold (float, optional): The metric threshold value.
            label (str, optional): The label associated with the rule.
            conf_threshold (float, optional): The confidence threshold.
            iou_threshold (float, optional): The intersection-over-union threshold.
            weights (dict, optional): Weights for the rule.
            type_ (str, optional): The type of the rule.
            background_label (str, optional): The background label.
            include_background (bool, optional): Whether to include background.
            
        Returns:
            None
        """
        from raga import REQUIRED_ARG_V2

        assert isinstance(metric, str) and metric, f"{REQUIRED_ARG_V2.format('metric', 'str')}"
        assert isinstance(metric_threshold, float) and metric_threshold, f"{REQUIRED_ARG_V2.format('metric_threshold', 'float')}"
        rules = {"metric" : metric, "threshold": metric_threshold}
        if iou_threshold: rules.update({"iou":iou_threshold })
        if conf_threshold: rules.update({"confThreshold":conf_threshold })
        if label: rules.update({"clazz":[label] })
        if weights: rules.update({"weights": weights})
        if type: rules.update({"type":type })
        if background_label: rules.update({"backgroundLabel":background_label })
        if background_label: rules.update({"includeBackground":include_background })
        self.rules.append(rules)
            

    def get(self) -> List[Dict[str, Any]]:
        """
        Get the list of rules.

        Returns:
            List[Dict[str, Any]]: The list of rules.
        """
        return self.rules
    
class LQRules():
    def __init__(self):
        self.rules = []

    def add(self, metric:str, metric_threshold:float, label:list):
        from raga import REQUIRED_ARG_V2
        assert isinstance(metric, str) and metric, f"{REQUIRED_ARG_V2.format('metric', 'str')}"
        assert isinstance(label, list) and label, f"{REQUIRED_ARG_V2.format('label', 'list')}"
        assert isinstance(metric_threshold, float) and metric_threshold, f"{REQUIRED_ARG_V2.format('metric_threshold', 'float')}"
        self.rules.append({ "metric" : metric, "threshold": metric_threshold,  "clazz": label})

    def get(self):
        return self.rules
    
class DriftDetectionRules():
    def __init__(self):
        self.rules = []

    def add(self, type:str, dist_metric:str, _class:str, threshold:float):
        from raga import REQUIRED_ARG_V2
        
        assert isinstance(dist_metric, str) and dist_metric, f"{REQUIRED_ARG_V2.format('dist_metric', 'str')}"
        assert isinstance(_class, str) and _class, f"{REQUIRED_ARG_V2.format('_class', 'str')}"
        assert isinstance(threshold, float) and threshold, f"{REQUIRED_ARG_V2.format('threshold', 'float')}"
        self.rules.append({ "type" : type, "dist_metric" : dist_metric,  "class": _class, "threshold":threshold })

    def get(self):
        return self.rules
    
class SemanticSegmentation:
    def __init__(self, 
                 Id:Optional[str], 
                 Format:Optional[str], 
                 Confidence:Optional[float], 
                 LabelId:Optional[str] = None, 
                 LabelName:Optional[str]=None, 
                 Segmentation:Optional[list]=None):
        self.Id = Id
        self.LabelId = LabelId
        self.LabelName = LabelName
        self.Segmentation = Segmentation
        self.Format = Format
        self.Confidence = Confidence

class SemanticSegmentationObject():
    def __init__(self):
        self.segmentations = list()
    
    def add(self, segmentations:SemanticSegmentation):
        self.segmentations.append(segmentations.__dict__)
    
    def get(self):
        return self.__dict__
    
class MistakeScore:
    def __init__(self):
        self.mistake_scores = dict()
    
    def add(self, key:(str, int), value:(float, int, str)):
        from raga import REQUIRED_ARG_V2
        assert isinstance(key, (str, int)) and key, f"{REQUIRED_ARG_V2.format('key', 'str or int')}"
        assert isinstance(value, (float, int, str)) and value is not None, f"{REQUIRED_ARG_V2.format('value', 'float or int or str')}"
        self.mistake_scores[key]=value
    
    def get(self):
        return self.__dict__

    
class ObjectDetection:
    def __init__(self, Id:Optional[str], Format:Optional[str], Confidence:Optional[float], ClassId:Optional[str] = None, ClassName:Optional[str]=None, BBox=None):
        self.Id = Id
        self.ClassId = ClassId
        self.ClassName = ClassName
        self.BBox = BBox
        self.Format = Format
        self.Confidence = Confidence

class VideoFrame:
    def __init__(self, frameId:Optional[str], timeOffsetMs:Optional[float], detections:ObjectDetection):
        self.frameId = frameId
        self.timeOffsetMs = timeOffsetMs
        self.detections = detections.__dict__.get('detections')

class ImageDetectionObject():
    def __init__(self):
        self.detections = list()
    
    def add(self, object_detection:ObjectDetection):
        self.detections.append(object_detection.__dict__)
    
    def get(self):
        return self.__dict__
    
class VideoDetectionObject():
    def __init__(self):
        self.frames = list()
    
    def add(self, video_frame:VideoFrame):
        self.frames.append(video_frame.__dict__)
    
    def get(self):
        return self.__dict__
    
class ImageClassificationElement():
    def __init__(self):
        self.confidence = dict()
    
    def add(self, key:(str, int), value:(float, int, str)):
        from raga import REQUIRED_ARG_V2
        assert isinstance(key, (str, int)) and key, f"{REQUIRED_ARG_V2.format('key', 'str or int')}"
        assert isinstance(value, (float, int, str)) and value is not None, f"{REQUIRED_ARG_V2.format('value', 'float or int or str')}"
        self.confidence[key]=value
    
    def get(self):
        return self.__dict__

class Embedding:
    def __init__(self, embedding_val: float):
        from raga import REQUIRED_ARG_V2
        assert isinstance(embedding_val, (float, int)), f"{REQUIRED_ARG_V2.format('embedding', 'float')}"
        self.embedding = embedding_val

class ImageEmbedding:
    def __init__(self):
         self.embeddings = []

    def add(self, embedding_values: Embedding):
        self.embeddings.append(embedding_values.embedding)

    def get(self):
        return self.__dict__

class ROIEmbedding:
    def __init__(self):
         self.embeddings = dict()

    def add(self, id,  embedding_values: list):
        self.embeddings[id] = embedding_values

    def get(self):
        return self.__dict__
    


from typing import Optional
from raga import TestSession, ModelABTestRules, FMARules, LQRules

def model_ab_test(test_session:TestSession, 
                  dataset_name: str, 
                  test_name: str, 
                  modelA: str, 
                  modelB: str,
                  type: str, 
                  rules: ModelABTestRules, 
                  aggregation_level:  Optional[list] = [],
                  gt: Optional[str] = "", 
                  filter: Optional[str] = ""):
    dataset_id = ab_test_validation(test_session, dataset_name, test_name, modelA, modelB, type, rules, gt, aggregation_level)    
    return {
            "datasetId": dataset_id,
            "experimentId": test_session.experiment_id,
            "name": test_name,
            "modelA": modelA,
            "modelB": modelB,
            "type": type,
            "rules": rules.get(),
            "aggregationLevels": aggregation_level,
            'filter':filter,
            'gt':gt,
            'test_type':'ab_test'
        }

def ab_test_validation(test_session:TestSession, 
                       dataset_name: str, 
                       test_name: str, 
                       modelA: str, 
                       modelB: str,
                       type: str, 
                       rules: ModelABTestRules,
                       gt: Optional[str] = "", 
                       aggregation_level:Optional[list] = []):
    from raga.constants import INVALID_RESPONSE, INVALID_RESPONSE_DATA, REQUIRED_ARG_V2

    assert isinstance(test_session, TestSession), f"{REQUIRED_ARG_V2.format('test_session', 'instance of the TestSession')}"
    assert isinstance(dataset_name, str) and dataset_name, f"{REQUIRED_ARG_V2.format('dataset_name', 'str')}"

    res_data = test_session.http_client.get(f"api/dataset?projectId={test_session.project_id}&name={dataset_name}", headers={"Authorization": f'Bearer {test_session.token}'})

    if not isinstance(res_data, dict):
            raise ValueError(INVALID_RESPONSE)
    
    dataset_id = res_data.get("data", {}).get("id")

    if not dataset_id:
        raise KeyError(INVALID_RESPONSE_DATA)
    
    assert isinstance(test_name, str) and test_name, f"{REQUIRED_ARG_V2.format('test_name', 'str')}"
    assert isinstance(modelA, str) and modelA, f"{REQUIRED_ARG_V2.format('modelA', 'str')}"
    assert isinstance(modelB, str) and modelB, f"{REQUIRED_ARG_V2.format('modelB', 'str')}"
    assert isinstance(type, str), f"{REQUIRED_ARG_V2.format('type', 'str')}"
    assert isinstance(rules, ModelABTestRules) and rules.get(), f"{REQUIRED_ARG_V2.format('rules', 'instance of the ModelABTestRules')}"

    if aggregation_level:
        assert isinstance(aggregation_level, list), f"{REQUIRED_ARG_V2.format('aggregation_level', 'str')}"

    if type == "labelled":
        assert isinstance(gt, str) and gt, f"{REQUIRED_ARG_V2.format('gt', 'str')}"

    if type == "unlabelled" and isinstance(gt, str) and gt:
        raise ValueError("gt is not required on unlabelled type.")
    
    return dataset_id

def failure_mode_analysis(test_session:TestSession, 
                          dataset_name:str, 
                          test_name:str, 
                          model:str, 
                          gt:str,
                          rules:FMARules, 
                          output_type:str, 
                          type:str, 
                          clustering:Optional[dict]={}, 
                          aggregation_level:Optional[list]=[]):
    
    dataset_id = failure_mode_analysis_validation(test_session=test_session, dataset_name=dataset_name, test_name=test_name, model=model, gt=gt, type=type, rules=rules, output_type=output_type, aggregation_level=aggregation_level, clustering=clustering)
   
    response = {
            "datasetId": dataset_id,
            "experimentId": test_session.experiment_id,
            "name": test_name,
            "model": model,
            "gt": gt,
            "type": type,
            "rules": rules.get(),
            "test_type":"cluster",
            "filter":"",
            "outputType":output_type,
            "aggregationLevels":aggregation_level,
        }
    if clustering:
         response['clustering'] = clustering
    return response

def failure_mode_analysis_validation(test_session:TestSession, dataset_name:str, test_name:str, model:str, gt:str, rules:FMARules, output_type:str, type:str, aggregation_level:Optional[list]=[], clustering:Optional[dict]=None):
    from raga.constants import INVALID_RESPONSE, INVALID_RESPONSE_DATA, REQUIRED_ARG_V2

    assert isinstance(test_session, TestSession), f"{REQUIRED_ARG_V2.format('test_session', 'instance of the TestSession')}"
    assert isinstance(dataset_name, str) and dataset_name, f"{REQUIRED_ARG_V2.format('dataset_name', 'str')}"
    res_data = test_session.http_client.get(f"api/dataset?projectId={test_session.project_id}&name={dataset_name}", headers={"Authorization": f'Bearer {test_session.token}'})

    if not isinstance(res_data, dict):
            raise ValueError(INVALID_RESPONSE)
    
    dataset_id = res_data.get("data", {}).get("id")

    if not dataset_id:
        raise KeyError(INVALID_RESPONSE_DATA)
    assert isinstance(test_name, str) and test_name, f"{REQUIRED_ARG_V2.format('test_name', 'str')}"
    assert isinstance(model, str) and model, f"{REQUIRED_ARG_V2.format('model', 'str')}"
    assert isinstance(gt, str) and gt, f"{REQUIRED_ARG_V2.format('gt', 'str')}"
    assert isinstance(type, str) and type, f"{REQUIRED_ARG_V2.format('type', 'str')}"
    assert isinstance(rules, FMARules) and rules, f"{REQUIRED_ARG_V2.format('rules', 'instance of the FMARules')}"
    assert isinstance(output_type, str) and output_type, f"{REQUIRED_ARG_V2.format('output_type', 'str')}"

    if output_type == "object_detection":
         if type == "embedding" and not clustering:
              raise ValueError(f"{REQUIRED_ARG_V2.format('clustering', 'clustering function')}")
         if type == "metadata":
            assert isinstance(aggregation_level, list) and aggregation_level, f"{REQUIRED_ARG_V2.format('aggregation_level', 'list')}"
    return dataset_id

def clustering(method:str, embedding_col:str, level:str, args=dict):
    from raga.constants import REQUIRED_ARG_V2

    assert isinstance(method, str) and method, f"{REQUIRED_ARG_V2.format('method', 'str')}"
    assert isinstance(embedding_col, str) and embedding_col, f"{REQUIRED_ARG_V2.format('embedding_col', 'str')}"
    assert isinstance(level, str) and level, f"{REQUIRED_ARG_V2.format('level', 'str')}"
    return {
        "method":method,
        "embeddingCol":embedding_col,
        "level":level,
        "args":args
    } 


def labelling_quality_test(test_session:TestSession, 
                           dataset_name:str, 
                           test_name:str, 
                           type:str, 
                           output_type: str, 
                           rules:LQRules, 
                           mistake_score_col_name: str):
    
    dataset_id = labelling_quality_test_validation(test_session, dataset_name, test_name, type, output_type, rules, mistake_score_col_name)
    return {
            "datasetId": dataset_id,
            "experimentId": test_session.experiment_id,
            "name": test_name,
            "type": type,
            "outputType": output_type,
            "rules": rules.get(),
            "mistakeScoreColName":mistake_score_col_name,
            "test_type":"labelling_quality",
            "filter":"",
        }


def labelling_quality_test_validation(test_session:TestSession, dataset_name:str, test_name:str, type:str, output_type:str,  rules:LQRules, mistake_score_col_name:str):
    from raga.constants import INVALID_RESPONSE, INVALID_RESPONSE_DATA, REQUIRED_ARG_V2

    assert isinstance(test_session, TestSession), f"{REQUIRED_ARG_V2.format('test_session', 'instance of the TestSession')}"
    assert isinstance(dataset_name, str) and dataset_name, f"{REQUIRED_ARG_V2.format('dataset_name', 'str')}"
    res_data = test_session.http_client.get(f"api/dataset?projectId={test_session.project_id}&name={dataset_name}", headers={"Authorization": f'Bearer {test_session.token}'})

    if not isinstance(res_data, dict):
            raise ValueError(INVALID_RESPONSE)
    dataset_id = res_data.get("data", {}).get("id")

    if not dataset_id:
        raise KeyError(INVALID_RESPONSE_DATA)
    
    assert isinstance(test_name, str) and test_name, f"{REQUIRED_ARG_V2.format('test_name', 'str')}"
    assert isinstance(type, str) and type, f"{REQUIRED_ARG_V2.format('type', 'str')}"
    assert isinstance(output_type, str) and output_type, f"{REQUIRED_ARG_V2.format('output_type', 'str')}"
    assert isinstance(mistake_score_col_name, str) and mistake_score_col_name, f"{REQUIRED_ARG_V2.format('mistake_score_col_name', 'str')}"
    assert isinstance(rules, LQRules) and rules, f"{REQUIRED_ARG_V2.format('rules', 'str')}"
    return dataset_id




from raga import *
import datetime
from raga._tests import failure_mode_analysis, clustering

run_name = f"loader_failure_mode_analysis_image_classification-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

test_session = TestSession(project_name="LivenessDetection", run_name= run_name, access_key="u4amNsYmYE0hQWCPPFQ8", secret_key="FUDZ3NxORAqyCCmY7eezJEU9gxX0uPM232V1g75W", host="http://13.126.220.245:8080")

rules = FMARules()
rules.add(metric="Precision", conf_threshold=0.5, metric_threshold=0.5, label="live")
rules.add(metric="Recall", conf_threshold=0.5, metric_threshold=0.5, label="live")
rules.add(metric="F1Score", conf_threshold=0.5, metric_threshold=0.5, label="live")

cls_default = clustering(method="k-means", embedding_col="ImageVectorsM1", level="image", args= {"numOfClusters": 9})

edge_case_detection = failure_mode_analysis(test_session=test_session,
                                            dataset_name = "live-dataset",
                                            test_name = "Test",
                                            model = "modelB",
                                            gt = "GT",
                                            rules = rules,
                                            type="embeddings",
                                            output_type="multi-label",
                                            clustering = cls_default)

test_session.add(edge_case_detection)

test_session.run()


"""Initialization file for the testing_platform package."""
import os
import logging

RAGA_CONFIG_FILE = ".raga/config"
DEBUG_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
# Get the value of the DEBUG environment variable
debug_mode = os.environ.get('DEBUG')

# Configure the logging format and level based on the DEBUG environment variable
if debug_mode:
    logging.basicConfig(
        format=DEBUG_FORMAT,
        level=logging.DEBUG
    )
else:
    logging.basicConfig(
        format=DEBUG_FORMAT,
        level=logging.INFO
    )
if debug_mode:
    # Add a file handler to log messages to a file
    file_handler = logging.FileHandler('debug.log')
    file_handler.setLevel(logging.DEBUG)
    file_handler.setFormatter(logging.Formatter(DEBUG_FORMAT))

    # Add the file handler to the logger
    logger = logging.getLogger(__name__)
    logger.addHandler(file_handler)

from .constants import *
from .test_session import *
from .dataset_creds import *
from .dataset import *
from .raga_schema import *
from ._tests import *
from .post_deployment_checks import *
from .model_executor_factory import ModelExecutorFactory


import os
import time
import requests
from typing import Optional
import pandas as pd
import logging
import zipfile


from raga.exception import RagaException
from raga.raga_schema import RagaSchema
from raga.validators.dataset_validations import DatasetValidator
from raga.dataset_creds import DatasetCreds
from raga import TestSession

from raga.utils import (get_file_name, 
                        delete_files, 
                        upload_file, 
                        create_csv_and_zip_from_data_frame, 
                        data_frame_extractor, 
                        make_arg, 
                        on_upload_success, 
                        on_upload_failed, 
                        check_key_value_existence)


logger = logging.getLogger(__name__)


class DatasetException(RagaException):
    pass


class Dataset:
    MAX_RETRIES = 3
    RETRY_DELAY = 1

    def __init__(
        self,
        test_session:TestSession,
        name: str,
        data: (pd.DataFrame, str) = None,
        schema: Optional[RagaSchema] = None,
        creds: Optional[DatasetCreds] = None,
        format: Optional[str] = None,
        model_name: Optional[str] = None,
        inference_col_name: Optional[str] = None,
        embedding_col_name: Optional[str] = None,
        ref_dataset: Optional[str] = None,
        u_test: bool = False
    ):
        self.test_session = test_session
        self.name = DatasetValidator.validate_name(name)
        self.creds = DatasetValidator.validate_creds(creds)
        self.csv_file = f"experiment_{test_session.experiment_id}_{self.name}.csv"
        self.zip_file = f"experiment_{test_session.experiment_id}_{self.name}.zip"
        self.dataset_id = None
        
        if not u_test:
            self.dataset_id = self.create_dataset()
            if self.creds and self.dataset_id:
                self.create_dataset_creds()

        self.dataset_file_id = None
        self.data_set_top_five_rows = {}
        self.raga_dataset = data
        self.raga_extracted_dataset = None
        self.raga_schema = schema
        self.dataset_schema_columns = None
        self.format = format
        self.model_name = model_name
        self.inference_col_name = inference_col_name
        self.embedding_col_name = embedding_col_name
        
        self.init()

    def init(self):
        from raga import EMPTY_DATA_FRAME, INVALID_DATA_ARG, INVALID_SCHEMA

        if isinstance(self.raga_dataset, str):
            self.load_dataset_from_file() # Assume it's a file path
        elif isinstance(self.raga_dataset, pd.DataFrame):
            if self.raga_dataset.empty:
                raise DatasetException(EMPTY_DATA_FRAME)
                
            if self.raga_schema is None:
                raise DatasetException(INVALID_SCHEMA)
            
            if not isinstance(self.raga_schema, RagaSchema):
                raise DatasetException(INVALID_SCHEMA)
            
            self.dataset_schema_columns = self.raga_schema.columns
            self.raga_extracted_dataset = data_frame_extractor(self.raga_dataset)  
        else:
            raise DatasetException(INVALID_DATA_ARG)

    def load(self,  schema: Optional[RagaSchema] = None):
        self.raga_schema = schema if schema else self.raga_schema
        retries = 0
        while retries < self.MAX_RETRIES:
            try:
                self.load_data_frame()
                    
                print("Data loaded successful!")
                break  # Exit the loop if initialization succeeds
            except requests.exceptions.RequestException as e:
                print(f"Network error occurred: {str(e)}")
                retries += 1
                if retries < self.MAX_RETRIES:
                    print(f"Retrying in {self.RETRY_DELAY} second(s)...")
                    time.sleep(self.RETRY_DELAY)

    def load_data_frame(self):
        """
        Loads the data frame, creates a CSV file, zips it, and uploads it to the server.
        """
        self.data_frame_validation()
        create_csv_and_zip_from_data_frame(self.raga_extracted_dataset, self.csv_file, self.zip_file)
        signed_upload_path, file_path = self.get_pre_signed_s3_url(self.zip_file)
        upload_file(
            signed_upload_path,
            self.zip_file,
            success_callback=on_upload_success,
            failure_callback=on_upload_failed,
        )
        delete_files(self.csv_file, self.zip_file)

        self.dataset_file_id = self.create_dataset_load_definition(file_path, "csv", self.raga_schema.columns)
        self.notify_server()

    def load_dataset_from_file(self):
        from raga import REQUIRED_ARG
        if not self.format:
            raise DatasetException(f"{REQUIRED_ARG.format('format')}")
        if not self.model_name:
            raise DatasetException(f"{REQUIRED_ARG.format('model_name')}")
        if not self.inference_col_name:
            raise DatasetException(f"{REQUIRED_ARG.format('inference_col_name')}")
            
        file_dir = os.path.dirname(self.raga_dataset)
        file_name_without_ext, file_extension, file_name = get_file_name(
            self.raga_dataset)
        zip_file_name = os.path.join(file_dir, file_name_without_ext + ".zip")

        with zipfile.ZipFile(zip_file_name, "w") as zip_file:
            zip_file.write(self.raga_dataset, file_name)
        signed_upload_path, file_path  = self.get_pre_signed_s3_url(
            file_name_without_ext + ".zip")
        
        upload_file(
            signed_upload_path,
            zip_file_name,
            success_callback=on_upload_success,
            failure_callback=on_upload_failed,
        )
        if os.path.exists(zip_file_name):
            os.remove(zip_file_name)
            logger.debug("Zip file deleted")
        else:
            logger.debug("Zip file not found")

        arguments = make_arg(self.model_name, self.inference_col_name, self.embedding_col_name)

        self.dataset_file_id = self.create_dataset_load_definition(file_path, format, arguments)
        self.notify_server()

    def head(self):
        res_data = self.test_session.http_client.get(
            f"api/dataset/{self.dataset_id}/data",
            headers={"Authorization": f'Bearer {self.test_session.token}'},
        )
        if not res_data or 'data' not in res_data or 'rows' not in res_data['data'] or 'columns' not in res_data['data']:
            raise DatasetException("Record not found!")
        
        print(self.filter_head(res_data.get('data', {}).get('rows', {}), res_data.get('data', {}).get('columns', {})))

    def filter_head(self, rows, columns):
        pd_data = pd.DataFrame(rows)
        columns_temp = []
        for col in columns:
            columns_temp.append(col.get("columnName"))
        return pd_data.get(columns_temp)

    def get_pre_signed_s3_url(self, file_name: str):
        res_data = self.test_session.http_client.get(
            "api/dataset/uploadpath",
            None,{"experimentId": self.test_session.experiment_id, "fileName": file_name, "contentType":"application/zip"},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )
        logger.debug("Pre-signed URL generated")
        print("Pre-signed URL generated")
        return res_data["data"]["signedUploadPath"], res_data["data"]["filePath"]

    def notify_server(self):
        """
        Notifies the server to load the dataset with the provided experiment ID and data definition.
        """
        res_data = self.test_session.http_client.post(
            "api/experiment/load-data",
            {"experimentId": self.test_session.experiment_id, "datasetFileId": self.dataset_file_id},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )
        logger.debug(res_data.get('data', ''))
        return res_data

    def create_dataset(self):
        if not self.test_session.project_id:
            raise DatasetException("Project ID is required.")
        if not self.test_session.token:
            raise DatasetException("Token is required.")

        res_data = self.test_session.http_client.post(
            "api/dataset",
            {"name": self.name, "projectId": self.test_session.project_id, "experimentId":self.test_session.experiment_id},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )

        if not res_data or 'data' not in res_data or 'id' not in res_data['data']:
            raise DatasetException("Failed to create dataset.")

        return res_data['data']['id']

    def create_dataset_creds(self,):
        if not self.dataset_id:
            raise DatasetException("Dataset ID is required.")
        if not self.test_session.token:
            raise DatasetException("Token is required.")

        data = {
            "datasetId": self.dataset_id,
            "storageService": "s3",
            "json": {'arn': self.creds.arn}
        }
        res_data = self.test_session.http_client.post(
            "api/dataset/credential",
            data,
            {"Authorization": f'Bearer {self.test_session.token}'},
        )

        if not res_data or 'data' not in res_data or 'id' not in res_data['data']:
            raise DatasetException("Failed to create dataset credentials.")

        return res_data['data']['id']
    
    def create_dataset_load_definition(self, filePath: str, type: str, arguments: dict):
        payload = {
            "datasetId": self.dataset_id,
            "filePath": filePath,
            "type": type,
            "arguments": arguments
        }

        res_data = self.test_session.http_client.post(
            "api/dataset/definition", payload,
            {"Authorization": f'Bearer {self.test_session.token}'},
        )
        return res_data.get('data',{}).get('id')

    def get_data_frame(self, columns:list):
        image_id_column = next((item['customerColumnName'] for item in self.raga_schema.columns if item['type'] == 'imageName'), None)
        if image_id_column not in columns:
            columns.append(image_id_column)
        
        missing_columns = [col for col in columns if col not in self.raga_extracted_dataset.columns]
        if not missing_columns:
            return self.raga_extracted_dataset[columns], image_id_column
        else:
            missing_columns_str = ', '.join(missing_columns)
            raise DatasetException(f"The following columns do not exist in the DataFrame: {missing_columns_str}")

    def set_data_frame(self, data_frame:pd.DataFrame):
        image_id_column = next((item['customerColumnName'] for item in self.raga_schema.columns if item['type'] == 'imageName'), None)
        merged_df = pd.merge(self.raga_extracted_dataset, data_frame, on=image_id_column, how='inner', suffixes=('', '_right'))
        self.raga_extracted_dataset = merged_df
        return 1
    
    def data_frame_validation(self):
        from raga.constants import SCHEMA_KEY_DOES_NOT_EXIST, COL_DOES_NOT_EXIST

        column_list = self.raga_extracted_dataset.columns.to_list()
        for col in self.raga_schema.columns:
            if col.get("customerColumnName") not in column_list:
                raise DatasetException(f"{COL_DOES_NOT_EXIST.format(col.get('customerColumnName'), column_list)}")
            self.validate_data_frame_value(col, self.raga_extracted_dataset.loc[0,col.get("customerColumnName")])

        if not check_key_value_existence(self.raga_schema.columns, 'type', 'imageName'):
            raise DatasetException(SCHEMA_KEY_DOES_NOT_EXIST)
    
    def validate_data_frame_value(self, col, col_value):
        from raga.constants import DATASET_FORMAT_ERROR

        if col.get('type') == "classification" and "confidence" not in col_value:
                raise DatasetException(f"{DATASET_FORMAT_ERROR.format(col.get('customerColumnName'))}")
        
        if (col.get('type') == "imageEmbedding" or col.get('type') == "roiEmbedding") and "embeddings" not in col_value:
                raise DatasetException(f"{DATASET_FORMAT_ERROR.format(col.get('customerColumnName'))}")
        
        if col.get('type') == "inference" and "detections" not in col_value:
                raise DatasetException(f"{DATASET_FORMAT_ERROR.format(col.get('customerColumnName'))}")
        return 1


import os
import unittest
from unittest import mock
import pandas as pd
from raga import upload_file, create_csv_and_zip_from_data_frame, delete_files, StringElement, data_frame_extractor
from raga.utils.dataset_util import FileUploadError

class TestFileUpload(unittest.TestCase):
    
    @mock.patch("raga.logger.debug")
    @mock.patch("os.remove")
    @mock.patch("os.path.exists")
    def test_delete_files_success(self, mock_path_exists, mock_remove, mock_logger_debug):
        mock_path_exists.side_effect = [True, True]
        mock_remove.return_value = None

        csv_file = "test.csv"
        zip_file = "test.zip"

        result = delete_files(csv_file, zip_file)

        mock_path_exists.assert_any_call(csv_file)
        mock_remove.assert_any_call(csv_file)
        mock_path_exists.assert_any_call(zip_file)
        mock_remove.assert_any_call(zip_file)
        self.assertTrue(result)

    @mock.patch("raga.logger.debug")
    @mock.patch("os.path.exists")
    def test_delete_files_csv_file_not_found(self, mock_path_exists, mock_logger_debug):
        mock_path_exists.return_value = False

        csv_file = "test.csv"
        zip_file = "test.zip"

        with self.assertRaises(FileNotFoundError) as context:
            delete_files(csv_file, zip_file)

        mock_path_exists.assert_called_with(csv_file)
        self.assertEqual(str(context.exception), "CSV file not found.")

    @mock.patch("raga.logger.debug")
    @mock.patch("os.remove")
    @mock.patch("os.path.exists")
    def test_delete_files_zip_file_not_found(self, mock_path_exists, mock_remove, mock_logger_debug):
        mock_path_exists.side_effect = [True, False]
        mock_remove.return_value = None

        csv_file = "test.csv"
        zip_file = "test.zip"

        with self.assertRaises(FileNotFoundError) as context:
            delete_files(csv_file, zip_file)

        mock_path_exists.assert_any_call(csv_file)
        mock_remove.assert_any_call(csv_file)
        mock_path_exists.assert_any_call(zip_file)
        self.assertEqual(str(context.exception), "Zip file not found.")


if __name__ == "__main__":
    unittest.main()



from raga import *

project_name = "testingProject"
run_name = "drift-7-aug-v4"

test_session = TestSession(project_name="testingProject",
                           run_name="drift-7-aug-v4",
                           u_test=True)
test_session.token = "eyJhbGciOiJIUzUxMiJ9.eyJzdWIiOiJhZG1pbkByYWdhIiwicm9sZXMiOlsiUk9MRV9BRE1JTiJdLCJ1c2VyTmFtZSI6ImFkbWluQHJhZ2EiLCJleHAiOjE2OTE1NTgwNTYsImlhdCI6MTY5MTQ3MTY1Niwib3JnSWQiOjEsImp0aSI6ImFkbWluQHJhZ2EifQ.jhDiidIdA2AFuNgWuFAwM2iTdXZHb7wvlaaM-nptakni6QvGh0QscFj-3tKTvEna1hqjNyx0mo9lhpnFtxgqOg"
test_session.project_id = 1
test_session.experiment_id = 1701

rules = DriftDetectionRules()
rules.add(type="anomaly_detection", dist_metric="Euclidian", _class="All", threshold=0.6)

edge_case_detection = data_drift_detection(test_session=test_session,
                                           test_name="dift-8-aug-v2",
                                           train_dataset_name="train-dataset-7-aug-v2",
                                           field_dataset_name="field-dataset-7-aug-v2",
                                           train_embed_col_name="ImageEmbedding",
                                           field_embed_col_name = "ImageEmbedding" ,
                                           level = "image",
                                           aggregation_level=["location", "vehicle_no", "date_captured", "channel", "dataset_type"],
                                           rules = rules)


test_session.add(edge_case_detection)

test_session.run()


from raga import *

test_session = TestSession(project_name="testingProject",
                           run_name="labelling-run-24-aug-v6")

rules = LQRules()
rules.add(metric="loss", label=["ALL"], metric_threshold=0.005)


edge_case_detection = labelling_quality_test(test_session=test_session,
                                            dataset_name = "satsure-dataset-aug-24-v1",
                                            test_name = "Test",
                                            type = "labelling_consistency",
                                            rules = rules)
test_session.add(edge_case_detection)

test_session.run()


import pathlib
from raga import *
import pandas as pd
import json
import datetime
import random

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def img_url(x):
    return StringElement(f"https://ragaaimedia.s3.ap-south-1.amazonaws.com/1/Hb.json/data_points/{pathlib.Path(x).stem}/{x}")

def model_a_inference(row):
    model_a_inference = VideoDetectionObject()
    for index, frame in enumerate(row["outputs"]):
        detections = ImageDetectionObject()
        for index, detection in enumerate(frame["detections"]):
            id = index+1
            detections.add(ObjectDetection(Id=id, Format="xywh_normalized", Confidence=detection["confidence"], ClassId=0, ClassName=detection["class"], BBox=detection["bbox"]))
        model_a_inference.add(VideoFrame(frameId=frame["frame_id"], timeOffsetMs=frame["time_offset_ms"], detections=detections))

    return model_a_inference


def model_b_inference(row):
    model_a_inference = VideoDetectionObject()
    for index, frame in enumerate(row["new_outputs"]):
        detections = ImageDetectionObject()
        for index, detection in enumerate(frame["detections"]):
            id = index+1
            detections.add(ObjectDetection(Id=id, Format="xywh_normalized", Confidence=detection["confidence"], ClassId=0, ClassName=detection["class"], BBox=detection["bbox"]))
        model_a_inference.add(VideoFrame(frameId=frame["frame_id"], timeOffsetMs=frame["time_offset_ms"], detections=detections))

    return model_a_inference

def json_parser(model1, model2):
    model1_df = pd.read_json(model1)
    model2_df = pd.read_json(model2)
    model1_df_exploded = model1_df.explode('inputs')
    model2_df_exploded = model2_df.explode('inputs')
    attributes = model2_df["attributes"].apply(pd.Series)
    model2_df_exploded = pd.concat([model2_df_exploded, attributes], axis=1)
    model2_df_exploded.rename(columns={"outputs": "new_outputs"}, inplace=True)
    merged_df = pd.merge(model1_df_exploded, model2_df_exploded, on='inputs').head(10)
    data_frame = pd.DataFrame()
    data_frame["videoId"] = merged_df["inputs"].apply(lambda x: StringElement(x))
    data_frame["videoUri"] = merged_df["inputs"].apply(lambda x: img_url(x))
    data_frame["timeOfCapture"] = merged_df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["sourceLink"] = merged_df["inputs"].apply(lambda x: StringElement(x))
    data_frame["dutyType"] = merged_df["dutyType"].apply(lambda x: StringElement(x))
    data_frame["timeOfDay"] = merged_df["time_of_day"].apply(lambda x: StringElement(x))
    data_frame["weather"] = merged_df["weather"].apply(lambda x: StringElement(x))
    data_frame["scene"] = merged_df["scene"].apply(lambda x: StringElement(x))
    data_frame["tags"] = merged_df["tags"].apply(lambda x: StringElement(x))
    data_frame["modelAInference"] = merged_df.apply(model_a_inference, axis=1)
    data_frame["modelBInference"] = merged_df.apply(model_b_inference, axis=1)
    return data_frame




pd_data_frame = json_parser("assets/model1.json", "assets/model2.json")

# print(data_frame_extractor(pd_data_frame).to_csv("assets/testing_data_frame_video_10.csv"))


schema = RagaSchema()
schema.add("videoId", PredictionSchemaElement())
schema.add("videoUri", ImageUriSchemaElement())
schema.add("timeOfCapture", TimeOfCaptureSchemaElement())
schema.add("sourceLink", FeatureSchemaElement())
schema.add("dutyType", AttributeSchemaElement())
schema.add("timeOfDay", AttributeSchemaElement())
schema.add("weather", AttributeSchemaElement())
schema.add("scene", AttributeSchemaElement())
schema.add("tags", AttributeSchemaElement())
schema.add("modelAInference", VideoInferenceSchemaElement(model="modelA"))
schema.add("modelBInference", VideoInferenceSchemaElement(model="modelB"))

run_name = f"lm_video_loader_failure_mode_analysis_object_detection-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="nnXvot82D3idpraRtCjJ", secret_key="P2doycL4WBZXLNARIs4bESxttzF3MHSC5K15Jrs9", host="http://65.0.13.122:8080")

creds = DatasetCreds(arn="arn:aws:iam::596708924737:role/test-s3")
#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="lm-hb-video-ds-v1", data=pd_data_frame, schema=schema, creds=creds, ref_dataset="lm-hb-v1")

#load schema and pandas data frame
test_ds.load()


from raga import *
import pandas as pd
import json
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def convert_json_to_data_frame(embeddings_file, dataset):
    with open(embeddings_file, 'r') as json_file:
        # Load JSON data
        embeddings_json = json.load(json_file)
    test_data_frame = []
    hr = 1
    for item in embeddings_json:
        attr = item.get('attributes', {})
        attributes_dict = {}
        for key, value in attr.items():
            attributes_dict[key] = StringElement(value)
        if attr.get("dataset_type", None) == dataset:
            print(dataset)
            ImageVectorsM1 = ImageEmbedding()
            image_embeddings = item.get("image_embedding", {})
            for value in image_embeddings:
                ImageVectorsM1.add(Embedding(value))

            file_name = os.path.basename(item["inputs"][0])
            data_point = {
                'ImageId': StringElement(file_name),
                'ImageUri': StringElement(f"https://nusceneraga.s3.ap-south-1.amazonaws.com/nuscene_rainy_images/{file_name}"),
                'TimeOfCapture': TimeStampElement(get_timestamp_x_hours_ago(hr)),
                'SourceLink': StringElement(file_name),
                'ImageEmbedding': ImageVectorsM1,
            }
            merged_dict = {**data_point, **attributes_dict}
            test_data_frame.append(merged_dict)
            hr+=1
            
    return test_data_frame
# convert_json_to_data_frame("nuscene_US_rainy_clear_engg_embedding.json", "field_data/train-data")
# print(len(convert_json_to_data_frame("ModelA_faster_rcnn_r50_fpn_nuscene_US_rainy_clear_engg_embedding.json", "train-data")))


#Convert JSON dataset to pandas Data Frame

pd_data_frame = pd.DataFrame(convert_json_to_data_frame("assets/nuscene_US_rainy_clear_engg_embedding.json", "field_data")).to_pickle('field-data-nuscene_US_rainy_clear_engg_embedding.pkl')

# # data_frame_extractor(pd_data_frame).to_csv("TestingDataFrame.csv", index=False)
dataset_ = "field"
version = "v1"
pd_data_frame = pd.read_pickle(f"{dataset_}-data-nuscene_US_rainy_clear_engg_embedding.pkl")

schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement(), pd_data_frame)
schema.add("ImageUri", ImageUriSchemaElement(), pd_data_frame)
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_data_frame)
schema.add("SourceLink", FeatureSchemaElement(), pd_data_frame)
schema.add("location", AttributeSchemaElement(), pd_data_frame)
schema.add("vehicle_no", AttributeSchemaElement(), pd_data_frame)
schema.add("date_captured", AttributeSchemaElement(), pd_data_frame)
schema.add("channel", AttributeSchemaElement(), pd_data_frame)
schema.add("dataset_type", AttributeSchemaElement(), pd_data_frame)
schema.add("ImageEmbedding", ImageEmbeddingSchemaElement(model="imageModel"), pd_data_frame)

#create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject",run_name= f"{dataset_}-run-21-aug-{version}")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name=f"{dataset_}-dataset-21-aug-{version}")

#load schema and pandas data frame
test_ds.load(data=pd_data_frame, schema=schema)


from raga import *

#create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", #Project Name
                           run_name="test-8-aug-v2")
#create payload for model ab testing
rules = ModelABTestRules() 
rules.add(metric="precision_diff", IoU=0.5, _class="All", threshold=0.5)
rules.add(metric="f1_score", IoU=0.5, _class="All", threshold=0.5)
rules.add(metric="recall_diff", IoU=0.5, _class="All", threshold=0.5)

model_comparison_check = model_ab_test(test_session=test_session, 
                                       dataset_name="dataset-7-aug-v3",
                                       test_name="AB-test-8-aug-v3",
                                       modelA = "modelA", 
                                       modelB = "modelB" ,
                                       type = "labelled", 
                                       gt="GT",
                                       rules = rules,
                                       aggregation_level=["location", "vehicle_no", "date_captured", "channel", "dataset_type"])

# add payload into test_session object
test_session.add(model_comparison_check)

# #run added ab test model payload
test_session.run()


import ast
from raga import *
import pandas as pd
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def image_classification(x):
    classification = ImageClassificationElement()
    x = ast.literal_eval(x)
    for key, value in x.items():
        classification.add(key=key, value=value)
    return classification

def mistake_score(x):
    mistake_score = MistakeScore()
    x = ast.literal_eval(x)
    for key, value in x.items():
        mistake_score.add(key=key, value=value)
    return mistake_score

def image_id(row):
    file = row["SourceLink"].split("/")[-1]
    return StringElement(f"{row['ImageId']}_{file}".replace(" ", "_"))

def csv_parser(csv_file):
    df = pd.read_csv(csv_file)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df.apply(image_id , axis=1)
    data_frame["ImageUri"] = df["SourceLink"].apply(lambda x: StringElement(f"https://raga-testing-storage.s3.ap-south-1.amazonaws.com/1{x.replace('100_sports_dataset', '')}"))
    data_frame["SourceLink"] = df["SourceLink"].apply(lambda x: StringElement(x))
    data_frame["TimeOfCapture"] = df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["GroundTruth"] = df["Ground Truth"].apply(image_classification)
    data_frame["MistakeScore"] = df["mistake_score"].apply(mistake_score)
    return data_frame

pd_data_frame = csv_parser("./assets/labelling_qc_score_df.csv")

#### Want to see in csv file uncomment line below ####
# data_frame_extractor(pd_data_frame).to_csv("./assets/labelling_qc_score_df_test.csv", index=False)

schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement())
schema.add("SourceLink", FeatureSchemaElement())
schema.add("GroundTruth", ImageClassificationSchemaElement(model="GT"))
schema.add("MistakeScore", MistakeScoreSchemaElement(ref_col_name="GroundTruth"))

run_name = f"loader_labeling_quality_image_classification-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="dE4Zs3iT8LnYPP3KNNsq", secret_key="JzDgIoB1irU9V2vbHtvTSNLvRzAzjbJZab2cZLos", host="http://13.126.220.245:8080")

cred = DatasetCreds(arn="arn:aws:iam::527593518644:role/s3-role")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="100-sport-dataset-v1", data=pd_data_frame, schema=schema, creds=cred)

#load to server
test_ds.load()


import logging
import os
import subprocess
import sys
import requests
from urllib.parse import urlparse, unquote
from raga.constants import INVALID_DATASET, INVALID_EXECUTION_ARG, INVALID_INIT_ARG, INVALID_RESPONSE, INVALID_RESPONSE_DATA, INVALID_RESPONSE_MODEL_ID_NOT_FOUND, MODEL_LIB_NOT_FOUND, MODEL_OR_PYTHON_NOT_FOUND, MODEL_REQUIRED_NOT_EMPTY, MODEL_VERSION_REQUIRED_NOT_EMPTY, PLATFORM_NOT_SUPPORT, PYTHON_NOT_SUPPORT, WHEEL_INSTALL_FAIL, WHEEL_NOT_FOUND, WHEEL_UNINSTALL_FAIL


from raga.dataset import Dataset
from raga.exception import RagaException
from raga.test_session import TestSession
from raga.utils.raga_config_reader import format_python_versions, get_machine_platform, get_python_version, get_config_file_path

logger = logging.getLogger(__name__)

RAGA_REPO_PATH = ".raga/raga_repo"
MODEL_PATH = "models"

class ModelExecutorFactoryException(RagaException):
    pass

class PlatformError(RagaException):
    pass

class PythonVersionError(RagaException):
    pass

class WheelFileInstallationError(RagaException):
    pass

class ModelExecutorError(RagaException):
    pass

class ModelExecutor:
    def __init__(self, executor, wheel_path):
        self.executor = executor
        self.wheel_file_path = wheel_path

    def execute(self, init_args, execution_args, data_frame:Dataset):
        execution_validation(init_args=init_args, execution_args=execution_args, dataset=data_frame)
        try:
            auth = {'aws_access_key_id': data_frame.test_session.aws_raga_access, 'aws_secret_access_key': data_frame.test_session.aws_raga_secret}
            role = data_frame.test_session.aws_raga_arn
            self.executor.initialise(init_args, role, auth)
            columns = list(execution_args.get('input_columns').values())
            column_keys = list(execution_args.get('output_columns').keys())
            column_schema = execution_args.get('column_schemas')
            for item in column_keys:
                col = execution_args['output_columns'][item]
                data_frame.raga_schema.add(col, column_schema[item])
            df = data_frame.get_data_frame(columns)[0]
            role = data_frame.creds.arn
            df = self.executor.run(data_frame=df, input_args=execution_args['input_columns'], output_args=execution_args['output_columns'], role=role, auth=auth)
            data_frame.set_data_frame(df)
            uninstall_wheel(self.wheel_file_path)
            return df
        except Exception as exc:
            logger.exception(exc)
            uninstall_wheel(self.wheel_file_path)
            sys.exit(1)


class ModelExecutorFactory:
    @classmethod
    def get_model_executor(cls, test_session:TestSession, model_name:str, version:str):
        if not isinstance(model_name, str) or not model_name:
            raise ModelExecutorFactoryException(MODEL_REQUIRED_NOT_EMPTY)
        if not isinstance(version, str) or not version:
            raise ModelExecutorFactoryException(MODEL_VERSION_REQUIRED_NOT_EMPTY)
        
        model_api_client = ModelAPIClient(test_session)

        local_platform = get_machine_platform()
        local_python_version = get_python_version()
        
        model_id = model_api_client.get_model_id(model_name=model_name)
        model_version = model_api_client.get_version_by_version(model_id=model_id, version=version, platform=local_platform, python=local_python_version)

        model_wheel =  model_validation(model_version, model_name, RAGA_REPO_PATH, MODEL_PATH)
        wheel_path = download_model(model_wheel)
        install_wheel(wheel_path)
        try:
            from raga_models.executor import Executor
            executor = Executor()
            return ModelExecutor(executor, wheel_path)
        except ImportError:
            raise ModelExecutorError(MODEL_LIB_NOT_FOUND)
    

    
class ModelAPIClient:
    def __init__(self, test_session=TestSession):
        self.http_client = test_session.http_client

        self.token = test_session.token
        self.project_id = test_session.project_id
        self.experiment_id = test_session.experiment_id


    def get_model_id(self, model_name):
        res_data = self.http_client.get(
            "api/model",
            params={"modelName": model_name, "projectId":self.project_id},
            headers={"Authorization": f'Bearer {self.token}'},
        )

        if not isinstance(res_data, dict):
            raise ValueError(INVALID_RESPONSE)

        model_id = res_data.get("data", {}).get('id')

        if not model_id:
            raise KeyError(INVALID_RESPONSE_MODEL_ID_NOT_FOUND)
        return model_id
    

    def get_version_by_version(self, model_id, version, platform, python):
        res_data = self.http_client.get(
            "api/models-version",
            params={
                "modelId": model_id,
                "version":version,
                "pythonVersion":python, 
                "platform":platform
                },
            headers={"Authorization": f'Bearer {self.token}'},
        )

        if not isinstance(res_data, dict):
            raise ValueError(INVALID_RESPONSE)

        data = res_data.get("data", {})

        if not data:
            raise KeyError(INVALID_RESPONSE_DATA)
        return data
    

def execution_validation(init_args: dict, execution_args: dict, dataset: Dataset):
    if not isinstance(dataset, Dataset):
        raise ModelExecutorFactoryException(INVALID_DATASET)
    
    if not isinstance(init_args, dict):
        raise ModelExecutorFactoryException(INVALID_INIT_ARG)
    
    if not isinstance(execution_args, dict):
        raise ModelExecutorFactoryException(INVALID_EXECUTION_ARG)
    return 1


def model_validation(model, model_name, raga_repo_path, model_path):

    if not model.get("wheelFile"):
        raise ModelExecutorFactoryException(WHEEL_NOT_FOUND)

    raga_models_path = os.path.join(get_config_file_path(raga_repo_path), f"{model_path}/{model_name}/{model.get('version')}")
    if not os.path.exists(raga_models_path):
        os.makedirs(raga_models_path)
    model["raga_models_path"] = raga_models_path
    whl_files = os.listdir(raga_models_path)
    if any(file.endswith('.whl') for file in whl_files): 
        model["whl_path"] = raga_models_path
    return model

def get_wheel_file_name(model):
    wheel_file_url = model.get("wheelFile")
    parts = wheel_file_url.split()
    file_path = parts[-1]
    parsed_file_path = urlparse(file_path)
    return unquote(parsed_file_path.path.split("/")[-1])

def download_model(model):
    wheel_file = model.get("wheelFile")
    if model.get("whl_path"):
        model_wheel = os.path.join(model.get("whl_path"), get_wheel_file_name(model))
    else:
        model_wheel = os.path.join(model.get('raga_models_path'),get_wheel_file_name(model))
        with requests.get(wheel_file, stream=True) as response:
            response.raise_for_status()
            with open(model_wheel, "wb") as f:
                for chunk in response.iter_content(chunk_size=8192):
                    f.write(chunk)
    return model_wheel


def install_wheel(package_name):
    try:
        logger.debug(f"PIP INSTALLING {package_name}")
        subprocess.run([sys.executable, "-m", "pip", "install", package_name], check=True, capture_output
        =True)
        logger.debug(f"PIP INSTALLED {package_name}")
    except subprocess.CalledProcessError as e:
        raise WheelFileInstallationError(f"{WHEEL_INSTALL_FAIL} {package_name}. Error: {e}")

def uninstall_wheel(package_name):
    try:
        logger.debug(f"PIP UNINSTALLING {package_name}")
        subprocess.run([sys.executable, "-m", "pip", "uninstall", package_name, "-y"], check=True, capture_output=True)
        logger.debug(f"PIP UNINSTALLED {package_name}")
    except subprocess.CalledProcessError as e:
        raise WheelFileInstallationError(f"{WHEEL_UNINSTALL_FAIL} {package_name}. Error: {e}")


class DatasetCredsValidator:
    @staticmethod
    def validate_type(type: str) -> str:
        valid_types = ['AWS_S3', 'GCP', 'Azure']  # Add multiple types as required
        if type not in valid_types:
            raise ValueError(f"Invalid type. Must be one of {valid_types}.")
        return type

    @staticmethod
    def validate_location(location: str) -> str:
        if not location:
            raise ValueError("Location is required.")
        # Additional validation logic specific to location, e.g., checking format or accessibility
        return location

    @staticmethod
    def validate_reference_id(reference_id: str) -> str:
        if not reference_id:
            raise ValueError("Reference ID is required.")
        # Additional validation logic specific to reference_id, e.g., checking length or format
        return reference_id


import unittest
from unittest.mock import patch, MagicMock
from urllib.parse import urlparse
from requests import Response
from requests.exceptions import RequestException
import json
import logging

from raga.utils import HTTPClient

class TestHTTPClient(unittest.TestCase):
    def setUp(self):
        self.base_url = "http://example.com"
        self.http_client = HTTPClient(self.base_url)

    def test_init(self):
        self.assertEqual(self.http_client.base_url, self.base_url)

    def test_validate_base_url_valid(self):
        valid_base_url = "http://example.com"
        validated_base_url = self.http_client.validate_base_url(valid_base_url)
        self.assertEqual(validated_base_url, valid_base_url)

    def test_validate_base_url_invalid(self):
        invalid_base_url = "example.com"
        with self.assertRaises(ValueError):
            self.http_client.validate_base_url(invalid_base_url)

    @patch("raga.utils.http_client.requests.get")
    @patch("raga.utils.http_client.logger.debug")
    def test_get_successful_request(self, mock_logger_debug, mock_requests_get):
        mock_response = MagicMock(spec=Response)
        mock_response.status_code = 200
        mock_response.json.return_value = {"success": True, "message": "Success message"}

        mock_requests_get.return_value = mock_response

        endpoint = "/api/get"
        params = {"param1": "value1"}
        data = {"key1": "value1"}
        headers = {"Content-Type": "application/json"}

        response = self.http_client.get(endpoint, params=params, data=data, headers=headers)

        mock_requests_get.assert_called_once_with(
            self.base_url + endpoint, params=params, data=json.dumps(data), headers=headers
        )
        
        self.assertEqual(response, {"success": True, "message": "Success message"})



    @patch("raga.utils.http_client.requests.get")
    def test_get_unsuccessful_request(self, mock_requests_get):
        mock_response = MagicMock(spec=Response)
        mock_response.status_code = 400
        mock_response.json.return_value = {"success": False, "message": "Error message"}

        mock_requests_get.return_value = mock_response

        endpoint = "/api/get"

        with self.assertRaises(ValueError) as cm:
            self.http_client.get(endpoint)

        mock_requests_get.assert_called_once_with(self.base_url + endpoint, params=None, data=None, headers={'Content-Type': 'application/json'})
        self.assertEqual(str(cm.exception), "Request failed with status code 400: Error message")

    @patch("raga.utils.http_client.requests.get")
    def test_get_request_exception(self, mock_requests_get):
        mock_requests_get.side_effect = RequestException()

        endpoint = "/api/get"

        with self.assertRaises(RequestException):
            self.http_client.get(endpoint)

        mock_requests_get.assert_called_once_with(self.base_url + endpoint, params=None, data=None, headers={'Content-Type': 'application/json'})

    @patch("raga.utils.http_client.requests.request")
    @patch("raga.utils.http_client.logger.debug")
    def test_post_successful_request(self, mock_logger_debug, mock_requests_request):
        mock_response = MagicMock(spec=Response)
        mock_response.status_code = 201
        mock_response.json.return_value = {"success": True, "message": "Success message"}

        mock_requests_request.return_value = mock_response

        endpoint = "/api/post"
        data = {"key1": "value1"}
        headers = {"Content-Type": "application/json"}

        response = self.http_client.post(endpoint, data=data, headers=headers)

        mock_requests_request.assert_called_once_with(
            "POST", self.base_url + endpoint, json=data, headers=headers
        )

        self.assertEqual(response, {"success": True, "message": "Success message"})

        
    @patch("raga.utils.http_client.requests.request")
    def test_post_unsuccessful_request(self, mock_requests_request):
        mock_response = MagicMock(spec=Response)
        mock_response.status_code = 400
        mock_response.json.return_value = {"success": False, "message": "Error message"}

        mock_requests_request.return_value = mock_response

        endpoint = "/api/post"

        with self.assertRaises(ValueError) as cm:
            self.http_client.post(endpoint)

        mock_requests_request.assert_called_once_with(
            "POST", self.base_url + endpoint, json=None, headers={'Content-Type': 'application/json'}
        )
        self.assertEqual(str(cm.exception), "Request failed with status code 400: Error message")

    @patch("raga.utils.http_client.requests.request")
    def test_post_request_exception(self, mock_requests_request):
        mock_requests_request.side_effect = RequestException()

        endpoint = "/api/post"

        with self.assertRaises(RequestException):
            self.http_client.post(endpoint)

        mock_requests_request.assert_called_once_with(
            "POST", self.base_url + endpoint, json=None, headers={'Content-Type': 'application/json'}
        )

if __name__ == "__main__":
    unittest.main()



import ast
from raga import *
import pandas as pd
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def replace_url(old_value):
    url = f"https://raga-testing-storage.s3.ap-south-1.amazonaws.com/1/Satellite Image dataset/{old_value.replace('./', '')}"
    return StringElement(url)

def replace_embedding(old_value):
    embeddings = ImageEmbedding()
    embeddings_list = ast.literal_eval(old_value)
    for embedding in embeddings_list:
        embeddings.add(Embedding(embedding))
    return embeddings

label_to_classname = {
    0: "Building",
	1: "Land",
	2: "Road",
	3: "Vegetation",
	4: "Water",
	5: "Unlabelled"
}

def csv_parser(csv_path):
    data_frame = pd.read_csv(csv_path)
    data_frame["ImageId"] = data_frame["ImageId"].apply(lambda x: StringElement(x))
    data_frame["ImageUri"] = data_frame['SourceLink'].apply(lambda x: replace_url(x))
    data_frame["TimeOfCapture"] = data_frame.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["Reflection"] = data_frame.apply(lambda row: StringElement("Yes"), axis=1)
    data_frame["Overlap"] = data_frame.apply(lambda row: StringElement("No"), axis=1)
    data_frame["CameraAngle"] = data_frame.apply(lambda row: StringElement("Yes"), axis=1)
    data_frame["SourceLink"] = data_frame['SourceLink'].apply(lambda x:StringElement(x.replace('./', '')))
    data_frame["Annotations"] = data_frame['Annotations'].apply(lambda x: replace_url(x))
    data_frame["ModelAInfernences"] = data_frame['ModelAInfernences'].apply(lambda x:replace_url(x))
    data_frame["ImageVectorsM1"] = data_frame['ImageVectorsM1'].apply(lambda x:replace_embedding(x))
    return data_frame


pd_data_frame = csv_parser("./assets/ImageSegementation.csv")

#### Want to see in csv file uncomment line below ####
# data_frame_extractor(pd_data_frame).to_csv("./assets/ImageSegementation_test.csv")

schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement())
schema.add("Reflection", AttributeSchemaElement())
schema.add("Overlap", AttributeSchemaElement())
schema.add("CameraAngle", AttributeSchemaElement())
schema.add("SourceLink", FeatureSchemaElement())
schema.add("Annotations", TIFFSchemaElement(label_mapping=label_to_classname, schema="tiff", model="ModelA"))
schema.add("ModelAInfernences", TIFFSchemaElement(label_mapping=label_to_classname, schema="tiff", model="ModelB"))
schema.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="imageModel"))




run_name = f"loader_failure_mode_analysis_semantic_segmentation-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="dE4Zs3iT8LnYPP3KNNsq", secret_key="JzDgIoB1irU9V2vbHtvTSNLvRzAzjbJZab2cZLos", host="http://3.111.106.226:8080")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="satellite_image_dataset_v3", data=pd_data_frame, schema=schema)

#load to server
test_ds.load()



# from raga import *
# import pandas as pd


# project_name = "testingProject" # Project Name
# run_name= "K_COCO_19Jul_V2_v9" # Experiment Name
# dataset_name = "k_coco_Ds_19jul_V2_v1" # Dataset Name


# #create test_session object of TestSession instance
# test_session = TestSession(project_name=project_name,run_name=run_name)

# #create test_ds object of Dataset instance
# test_ds = Dataset(test_session=test_session, name=dataset_name)

# #load schema and pandas data frame
# test_ds.load(data="macoco.json", format="coco", model_name="modelA", inference_col_name="annotations1")
# test_ds.load(data="mbcoco.json", format="coco", model_name="modelB", inference_col_name="annotations2")
# test_ds.load(data="gtcoco.json", format="coco", model_name="modelG", inference_col_name="annotations3")

# # Params for labelled AB Model Testing
# testName = StringElement("TestingP-unlabelled-test-1")
# modelA = StringElement("modelA")
# modelB = StringElement("modelB")
# gt = StringElement("modelG")
# type = ModelABTestTypeElement("labelled")
# rules = ModelABTestRules()
# rules.add(metric = StringElement("precision_diff_all"), IoU = FloatElement(0.5), _class = StringElement("ALL"), threshold = FloatElement(0.5))
# rules.add(metric = StringElement("precision_candy"), IoU = FloatElement(0.5), _class = StringElement("candy"), threshold = FloatElement(0.5))
# #create payload for model ab testing
# model_comparison_check = model_ab_test(test_session, dataset_name=dataset_name, testName=testName, modelA = modelA , modelB = modelB , type = type, rules = rules, gt=gt)

# #add payload into test_session object
# test_session.add(model_comparison_check)

# #run added ab test model payload
# test_session.run()

# # Params for unlabelled AB Model Testing
# testName = StringElement("19July_TestCOCO_KB_01")
# modelA = StringElement("modelA")
# modelB = StringElement("modelB")
# type = ModelABTestTypeElement("unlabelled")
# rules = ModelABTestRules()
# rules.add(metric = StringElement("difference_all"), IoU = FloatElement(0.5), _class = StringElement("ALL"), threshold = FloatElement(0.5))
# rules.add(metric = StringElement("difference_candy"), IoU = FloatElement(0.5), _class = StringElement("candy"), threshold = FloatElement(0.5))
# rules.add(metric = StringElement("difference_drink"), IoU = FloatElement(0.5), _class = StringElement("drink"), threshold = FloatElement(0.5))

# #create payload for model ab testing
# model_comparison_check = model_ab_test(test_session, dataset_name=dataset_name, testName=testName, modelA = modelA , modelB = modelB , type = type, rules = rules)

# #add payload into test_session object
# test_session.add(model_comparison_check)

# #run added ab test model payload
# test_session.run()


from raga import *
import datetime

run_name = f"run-labeling-quality-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="dE4Zs3iT8LnYPP3KNNsq", secret_key="JzDgIoB1irU9V2vbHtvTSNLvRzAzjbJZab2cZLos", host="http://13.126.220.245:8080")

rules = LQRules()
rules.add(metric="mistake_score", label=["ALL"], metric_threshold=0.0012)


edge_case_detection = labelling_quality_test(test_session=test_session,
                                            dataset_name = "satsure-full-dataset-v4",
                                            test_name = "Labeling Quality Test",
                                            type = "labelling_consistency",
                                            output_type="semantic_segmentation",
                                            mistake_score_col_name = "MistakeScores",
                                            rules = rules)
test_session.add(edge_case_detection)

test_session.run()


import unittest
from unittest.mock import MagicMock, patch
from unittest import mock
import pandas as pd
from raga import *
from raga.utils import on_upload_success, on_upload_failed
from raga.dataset import Dataset
class TestDataset(unittest.TestCase):

    def setUp(self):
        self.test_session = TestSession("project_name", "run_name", u_test=True)
        self.test_session.project_id = "project_id"
        self.test_session.token = "token"
        self.dataset_name = "my_dataset"
        self.test_session.experiment_id = "experiment_id"
        self.dataset_creds = None
        self.dataset = Dataset(self.test_session, self.dataset_name, self.dataset_creds, u_test=True)
        self.dataset.dataset_id = "12345"
        self.dataset.zip_file = "experiment_experiment_id.zip"

    def test_create_dataset(self):
        expected_dataset_id = "12345"
        mock_create_dataset = MagicMock(return_value={"data": {"id": expected_dataset_id}})
        self.test_session.http_client.post = mock_create_dataset

        dataset_id = self.dataset.create_dataset()

        self.assertEqual(dataset_id, expected_dataset_id)
        mock_create_dataset.assert_called_once_with(
            "api/dataset",
            {"name": self.dataset_name, "projectId": self.test_session.project_id, "experimentId":self.test_session.experiment_id},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )


    def test_get_pre_signed_s3_url(self):
        expected_signed_upload_path = "https://s3.amazonaws.com/my-bucket/upload"
        expected_file_path = "my-bucket/folder/file.zip"
        mock_get_pre_signed_url = MagicMock(return_value={"data": {"signedUploadPath": expected_signed_upload_path, "filePath": expected_file_path}})
        self.test_session.http_client.get = mock_get_pre_signed_url

        signed_upload_path, file_path = self.dataset.get_pre_signed_s3_url("file.zip")

        self.assertEqual(signed_upload_path, expected_signed_upload_path)
        self.assertEqual(file_path, expected_file_path)
        mock_get_pre_signed_url.assert_called_once_with(
            "api/dataset/uploadpath",
            None,
            {"experimentId": self.test_session.experiment_id, "fileName": "file.zip", "contentType":"application/zip"},
            {"Authorization": f'Bearer {self.test_session.token}'},
        )

    def test_create_dataset_load_definition(self):
        self.dataset.dataset_id = "dataset_id"
        file_path = "file_path"
        data_type = "pandas"
        arguments = {"arg1": "value1", "arg2": "value2"}
        mock_post = MagicMock(return_value={"data": {"id":"id"}})
        self.test_session.http_client.post = mock_post

        result = self.dataset.create_dataset_load_definition(file_path, data_type, arguments)

        mock_post.assert_called_once_with(
            "api/dataset/definition",
            {
                "datasetId": "dataset_id",
                "filePath": "file_path",
                "type": "pandas",
                "arguments": {"arg1": "value1", "arg2": "value2"},
            },
            {"Authorization": "Bearer token"},
        )
        self.assertEqual(result, "id")


    @mock.patch("raga.dataset.create_csv_and_zip_from_data_frame")
    @mock.patch("raga.dataset.upload_file")
    @mock.patch("raga.dataset.delete_files")
    @mock.patch("raga.dataset.Dataset.create_dataset_load_definition")
    @mock.patch("raga.dataset.Dataset.notify_server")
    @mock.patch("raga.dataset.Dataset.get_pre_signed_s3_url")
    def test_load_data_frame(self, mock_get_pre_signed_s3_url, mock_notify_server,
                             mock_create_dataset_load_definition, mock_delete_files,
                             mock_upload_file, mock_create_csv_and_zip_from_data_frame):
        data_frame = pd.DataFrame({"col1": [1, 2, 3], "col2": ["a", "b", "c"]})
        dataset_column = [{"name": "col1", "model": "", "type": "prediction", "description": ""}]
        csv_file = "experiment_experiment_id.csv"
        zip_file = "experiment_experiment_id.zip"
        signed_upload_path = "https://example.com/upload"
        file_path = "path/to/zip_file"

        mock_get_pre_signed_s3_url.return_value = signed_upload_path, file_path
        mock_create_csv_and_zip_from_data_frame.return_value = None
        mock_upload_file.return_value = None
        mock_delete_files.return_value = None
        mock_create_dataset_load_definition.return_value = None
        mock_notify_server.return_value = None

        self.dataset.load_data_frame(data_frame, dataset_column)

        mock_create_csv_and_zip_from_data_frame.assert_called_once_with(data_frame, csv_file, zip_file)
        mock_get_pre_signed_s3_url.assert_called_once_with(zip_file)
        mock_upload_file.assert_called_once_with(signed_upload_path, zip_file, success_callback=on_upload_success,
                                                 failure_callback=on_upload_failed)
        mock_delete_files.assert_called_once_with(csv_file, zip_file)
        mock_create_dataset_load_definition.assert_called_once_with( file_path, "csv", dataset_column)
        mock_notify_server.assert_called_once()

    @mock.patch("raga.dataset.Dataset.get_pre_signed_s3_url")
    @mock.patch("raga.dataset.upload_file")
    @mock.patch("raga.dataset.delete_files")
    def test_load_data_frame_upload_failure(
        self,
        mock_delete_files,
        mock_upload_file,
        mock_get_pre_signed_s3_url,
    ):
        data_frame = pd.DataFrame({"col1": [1, 2, 3], "col2": ["a", "b", "c"]})
        dataset_column = "dataset_column"

        mock_get_pre_signed_s3_url.return_value = ("signed_upload_path", "file_path")
        mock_upload_file.side_effect = Exception("Upload failed")
        mock_delete_files.return_value = None

        with self.assertRaises(Exception) as context:
            self.dataset.load_data_frame(data_frame, dataset_column)
            self.assertEqual(context.exception, "Upload failed")

            mock_get_pre_signed_s3_url.assert_called_once_with(self.dataset.zip_file)
            mock_upload_file.assert_called_once_with(
                "signed_upload_path",
                self.dataset.zip_file,
                success_callback=on_upload_success,
                failure_callback=on_upload_failed,
            )
            mock_delete_files.assert_called_once_with(
                self.dataset.csv_file, self.dataset.zip_file
            )



    @mock.patch('time.sleep')
    @mock.patch('builtins.print')
    def test_load_success(self, mock_print, mock_sleep):
        # Mock the necessary methods
        mock_initialize = MagicMock()
        self.dataset.initialize = mock_initialize

        # Call the method
        self.dataset.load(data='data', schema='schema', format='format', model_name='model_name',
                          inference_col_name='inference_col', embedding_col_name='embedding_col')

        # Assert the expected calls and prints
        mock_initialize.assert_called_once_with('data', 'schema', 'format', 'model_name',
                                                'inference_col', 'embedding_col')
        mock_print.assert_called_once_with("Data loaded successful!")
        mock_sleep.assert_not_called()

    @mock.patch('time.sleep')
    def test_load_network_error(self, mock_sleep):
       mock_initialize = MagicMock(side_effect=requests.exceptions.RequestException("Network error"))

       with mock.patch('raga.dataset.Dataset.initialize', side_effect=mock_initialize), \
                mock.patch('builtins.print') as mock_print:
            # Set the MAX_RETRIES to 3 for testing
            self.dataset.MAX_RETRIES = 3
            self.dataset.RETRY_DELAY = 1
            # Call the method
            self.dataset.load(data='data', schema='schema', format='format', model_name='model_name',
                          inference_col_name='inference_col', embedding_col_name='embedding_col')
            
            # Assert the expected prints and retries
            expected_calls = [
                mock.call('Network error occurred: Network error'),
                mock.call('Retrying in 1 second(s)...'),
                mock.call('Network error occurred: Network error'),
                mock.call('Retrying in 1 second(s)...'),
                mock.call('Network error occurred: Network error')
            ]
            mock_print.assert_has_calls(expected_calls)
            self.assertEqual(mock_initialize.call_count, 3)
            mock_sleep.assert_called_with(1)

    def test_load_key_error(self):
       mock_initialize = MagicMock(side_effect=KeyError("Key error"))

       with mock.patch('raga.dataset.Dataset.initialize', side_effect=mock_initialize), \
                mock.patch('builtins.print') as mock_print, \
                    self.assertRaises(SystemExit) as cm:
           
            # Call the method
            self.dataset.load(data='data', schema='schema', format='format', model_name='model_name',
                          inference_col_name='inference_col', embedding_col_name='embedding_col')
            
            # Assert the expected prints and retries
            mock_initialize.assert_called_once()
            mock_print.assert_called_with('Key error occurred: Key error')
            self.assertEqual(cm.exception.code, 1)
            
    def test_load_value_error(self):
        mock_initialize = MagicMock(side_effect=ValueError("Value error"))

        with mock.patch('raga.dataset.Dataset.initialize', side_effect=mock_initialize), \
                mock.patch('builtins.print') as mock_print, \
                    self.assertRaises(SystemExit) as cm:
           
            # Call the method
            self.dataset.load(data='data', schema='schema', format='format', model_name='model_name',
                          inference_col_name='inference_col', embedding_col_name='embedding_col')
            
            # Assert the expected prints and retries
            mock_initialize.assert_called_once()
            mock_print.assert_called_with('Value error occurred: Value error')
            self.assertEqual(cm.exception.code, 1)

    def test_load_unexpected_error(self):
        mock_initialize = MagicMock(side_effect=Exception("Unexpected error"))

        with mock.patch('raga.dataset.Dataset.initialize', side_effect=mock_initialize), \
                mock.patch('builtins.print') as mock_print, \
                    self.assertRaises(SystemExit) as cm:
           
            # Call the method
            self.dataset.load(data='data', schema='schema', format='format', model_name='model_name',
                          inference_col_name='inference_col', embedding_col_name='embedding_col')
            
            # Assert the expected prints and retries
            mock_initialize.assert_called_once()
            mock_print.assert_called_with('An unexpected error occurred: Unexpected error')
            self.assertEqual(cm.exception.code, 1)

if __name__ == "__main__":
    unittest.main()



import ast
import pandas as pd
import json

from raga import *

import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def img_url(x):
    return StringElement(f"https://raga-test-bucket.s3.ap-south-1.amazonaws.com/spoof/{x}")

def model_a_inference(row):
    classification = ImageClassificationElement()
    try:
        confidence = ast.literal_eval(row['ModelA Inference'])
        classification.add("live", confidence.get('live'))
    except Exception:
        classification.add("live", 0)
    return classification


def model_b_inference(row):
    classification = ImageClassificationElement()
    try:
        confidence = ast.literal_eval(row['ModelB Inference'])
        classification.add("live", confidence.get('live'))
    except Exception:
        classification.add("live", 0)
    return classification


def model_gt_inference(row):
    classification = ImageClassificationElement()
    try:
        confidence = ast.literal_eval(row['Ground Truth'])
        classification.add("live", confidence.get('live'))
    except Exception:
        classification.add("live", 0)
    return classification

def image_vectors_m1(row):
    image_vectors = ImageEmbedding()
    for embedding in json.loads(row['ImageVectorsM1']):
        image_vectors.add(Embedding(embedding))
    return image_vectors


def csv_parser(csv_file):
    df = pd.read_csv(csv_file).head(1)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df["ImageId"].apply(lambda x: StringElement(x))
    data_frame["ImageUri"] = df["ImageId"].apply(lambda x: img_url(x))
    data_frame["TimeOfCapture"] = df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["Reflection"] = df.apply(lambda row: StringElement("Yes"), axis=1)
    data_frame["Overlap"] = df.apply(lambda row: StringElement("No"), axis=1)
    data_frame["CameraAngle"] = df.apply(lambda row: StringElement("Yes"), axis=1)
    data_frame["SourceLink"] = df["SourceLink"].apply(lambda x: StringElement(f"/Users/manabroy/Downloads/retail dataset/spoof/{x.split('/')[-1]}"))
    data_frame["ModelA Inference"] = df.apply(model_a_inference, axis=1)
    data_frame["ModelB Inference"] = df.apply(model_b_inference, axis=1)
    data_frame["Ground Truth"] = df.apply(model_gt_inference, axis=1)
    data_frame["ImageVectorsM1"] =  df.apply(image_vectors_m1, axis=1)
    return data_frame


raga_data_frame = csv_parser("./assets/signzy_df.csv")


schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())

run_name = f"model_distribution-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"


test_session = TestSession(project_name="testingProject", run_name= run_name)

cred = DatasetCreds(arn="arn:aws:iam::527593518644:role/model-distribution")

raga_dataset = Dataset(test_session=test_session, 
                       name="md-fast-v2", 
                       data=raga_data_frame, 
                       schema=schema, 
                       creds=cred)
raga_dataset.load()


model_exe_fun = ModelExecutorFactory().get_model_executor(test_session=test_session, 
                                                          model_name="Signzy Embedding Model", 
                                                          version="0.1.9")

model_exe_fun.execute(init_args={"device": "cpu"}, 
                           execution_args={"input_columns":{"img_paths":"ImageUri"}, 
                                           "output_columns":{"embedding":"ImageEmbedding"},
                                           "column_schemas":{"embedding":ImageEmbeddingSchemaElement(model="Signzy Embedding Model")}}, 
                           data_frame=raga_dataset)

raga_dataset.load()


from typing import Optional
from raga import TestSession, DriftDetectionRules

def data_drift_detection(test_session:TestSession, 
                         train_dataset_name: str, 
                         field_dataset_name: str, 
                         test_name: str, 
                         train_embed_col_name: str, 
                         field_embed_col_name: str, 
                         level: str, 
                         rules:DriftDetectionRules, 
                         aggregation_level:  Optional[list] = [],
                         filter: Optional[str] = ""):
    train_dataset_id, field_dataset_id = data_drift_detection_validation(test_session, train_dataset_name, field_dataset_name, test_name, train_embed_col_name, field_embed_col_name, level, rules, aggregation_level)    

    return {
            "datasetId": field_dataset_id,
            "experimentId": test_session.experiment_id,
            "name": test_name,
            "aggregationLevels": aggregation_level,
            "filter":filter,
            "trainDatasetId":train_dataset_id,
            "trainEmbedColName": train_embed_col_name,
            "fieldEmbedColName": field_embed_col_name,
            "level": level,
            "rules": rules.get(),
            'test_type':'drift_test'
        }


def data_drift_detection_validation(test_session:TestSession, 
                                    train_dataset_name: str, 
                                    field_dataset_name: str,
                                    test_name: str,
                                    train_embed_col_name: str,
                                    field_embed_col_name: str,
                                    level: str, 
                                    rules=DriftDetectionRules,
                                    aggregation_level:Optional[list] = []):
    from raga.constants import INVALID_RESPONSE, INVALID_RESPONSE_DATA, REQUIRED_ARG_V2

    assert isinstance(test_session, TestSession), f"{REQUIRED_ARG_V2.format('test_session', 'instance of the TestSession')}"
    assert isinstance(train_dataset_name, str) and train_dataset_name, f"{REQUIRED_ARG_V2.format('train_dataset_name', 'str')}"
    assert isinstance(field_dataset_name, str) and field_dataset_name, f"{REQUIRED_ARG_V2.format('field_dataset_name', 'str')}"
    
    train_res_data = test_session.http_client.get(f"api/dataset?projectId={test_session.project_id}&name={train_dataset_name}", headers={"Authorization": f'Bearer {test_session.token}'})
    if not isinstance(train_res_data, dict):
            raise ValueError(INVALID_RESPONSE)
    train_dataset_id = train_res_data.get("data", {}).get("id")
    if not train_dataset_id:
        raise KeyError(INVALID_RESPONSE_DATA)
    
    field_res_data = test_session.http_client.get(f"api/dataset?projectId={test_session.project_id}&name={field_dataset_name}", headers={"Authorization": f'Bearer {test_session.token}'})
    if not isinstance(field_res_data, dict):
            raise ValueError(INVALID_RESPONSE)
    field_dataset_id = field_res_data.get("data", {}).get("id")
    if not field_dataset_id:
        raise KeyError(INVALID_RESPONSE_DATA)
    
    assert isinstance(test_name, str) and test_name, f"{REQUIRED_ARG_V2.format('test_name', 'str')}"
    assert isinstance(train_embed_col_name, str) and train_embed_col_name, f"{REQUIRED_ARG_V2.format('train_embed_col_name', 'str')}"
    assert isinstance(field_embed_col_name, str) and field_embed_col_name, f"{REQUIRED_ARG_V2.format('field_embed_col_name', 'str')}"
    assert isinstance(level, str) and level, f"{REQUIRED_ARG_V2.format('level', 'str')}"
    assert isinstance(rules, DriftDetectionRules) and rules.get(), f"{REQUIRED_ARG_V2.format('rules', 'instance of the DriftDetectionRules')}"

    if aggregation_level:
        assert isinstance(aggregation_level, list), f"{REQUIRED_ARG_V2.format('aggregation_level', 'list')}"

    return train_dataset_id, field_dataset_id



MODEL_REQUIRED_NOT_EMPTY = "model_name is required and must be a non-empty string."
MODEL_VERSION_REQUIRED_NOT_EMPTY = "version is required and must be a non-empty str."
MODEL_LIB_NOT_FOUND = "Model executor lib not found"
INVALID_RESPONSE = "Invalid response data. Expected a dictionary."
INVALID_RESPONSE_MODEL_ID_NOT_FOUND = "Invalid response data. model ID not found."
INVALID_RESPONSE_DATA = "Invalid response data."
INVALID_DATASET = "Invalid dataset."
INVALID_INIT_ARG = "Invalid init_args."
INVALID_EXECUTION_ARG = "Invalid execution_args."
MODEL_OR_PYTHON_NOT_FOUND = "Model platform or python version not found."
WHEEL_NOT_FOUND = "Wheel file not found."
WHEEL_UNINSTALL_FAIL = "Failed to uninstall"
WHEEL_INSTALL_FAIL = "Failed to install"
PLATFORM_NOT_SUPPORT = "The model version does does not support your platform. Please try other model version. Your platform {local_platform}. Model platform version {supported_platforms}."
PYTHON_NOT_SUPPORT = "The python version does does not support your python. Please try other model version. Your version {local_python_version}. Model Python version{supported_python_versions}."

INVALID_SCHEMA = "Invalid schema argument. Expected an instance of RagaSchema."
EMPTY_DATA_FRAME = "Empty DataFrame data argument."
INVALID_DATA_ARG = "Invalid data argument. Expected a DataFrame or a file path or schema."
SCHEMA_KEY_DOES_NOT_EXIST = "Missing required field 'ImageId' in the schema. Please provide 'ImageId' using a PredictionSchemaElement instance."
COL_DOES_NOT_EXIST = "Raga Schema Error: Column name `{0}` not found in provided Data Frame. {1}"

REQUIRED_ARG = "{0} argument is required."
REQUIRED_ARG_V2 = "{0} is required and must be a/an {1}."
DATASET_FORMAT_ERROR = "Invalid format found in '{0}' column."




import pathlib
from raga import *
import pandas as pd
import json
import datetime
import random

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def convert_json_to_data_frame(json_file_path_model_1, json_file_path_model_2, json_file_path_model_3, embeddings_file=None):
    test_data_frame = []
    with open(json_file_path_model_1, 'r') as json_file:
        # Load JSON data
        model_1 = json.load(json_file)
    with open(json_file_path_model_2, 'r') as json_file:
        # Load JSON data
        model_2 = json.load(json_file)
    
    with open(json_file_path_model_3, 'r') as json_file:
        # Load JSON data
        model_gt = json.load(json_file)
    if embeddings_file:
        with open(embeddings_file, 'r') as json_file:
            # Load JSON data
            embeddings_json = json.load(json_file)

    # Create a dictionary to store the inputs and corresponding data points
    inputs_dict_model_1 = {}
    hr = 1
    # Process model_1 data
    for item in model_1:
        inputs = item["inputs"]
        inputs_dict_model_1[tuple(inputs)] = item
    
    inputs_dict_model_3 = {}
    # Process model_3 data
    for item in model_gt:
        inputs = item["inputs"]
        inputs_dict_model_3[tuple(inputs)] = item

    
    inputs_dict_emb = {}
    # Process emb data
    for item in embeddings_json:
        inputs = item["inputs"]
        inputs_dict_emb[tuple(inputs)] = item

    # Process model_2 data
    for main_index, item in enumerate(model_2):

        ModelAInfernences = VideoDetectionObject()
        ModelBInfernences = VideoDetectionObject()
        
        inputs = item["inputs"]
        
        for index, frame in enumerate(item["outputs"]):
            detections = ImageDetectionObject()
            for index, detection in enumerate(frame["detections"]):
                id = index+1
                detections.add(ObjectDetection(Id=id, Format="xywh_normalized", Confidence=detection["confidence"], ClassId=0, ClassName=detection["class"], BBox=detection["bbox"]))
            ModelAInfernences.add(VideoFrame(frameId=frame["frame_id"], timeOffsetMs=frame["time_offset_ms"], detections=detections))
            

        attributes_dict = {}
        attributes = item.get("attributes", {})
        for key, value in attributes.items():
            attributes_dict[key] = StringElement(value)

        merged_item = inputs_dict_model_1.get(tuple(inputs), {})
        for index, frame in enumerate(merged_item["outputs"]):
            id = index+1
            detections = ImageDetectionObject()
            for index, detection in enumerate(frame["detections"]):
                id = index+1
                detections.add(ObjectDetection(Id=id, Format="xywh_normalized", Confidence=detection["confidence"], ClassId=0, ClassName=detection["class"], BBox=detection["bbox"]))
            ModelBInfernences.add(VideoFrame(frameId=frame["frame_id"], timeOffsetMs=frame["time_offset_ms"], detections=detections))
        
        merged_item = inputs_dict_model_3.get(tuple(inputs), {})
       
        for index, detection in enumerate(merged_item["outputs"][0]["detections"]):
            id = index+1
        
        merged_item = inputs_dict_emb.get(tuple(inputs), {})
        ImageVectorsM1 = ImageEmbedding()
        
        image_embeddings = merged_item.get("image_embedding", {})
        for value in image_embeddings:
            ImageVectorsM1.add(Embedding(value))

        file_name = os.path.basename(item["inputs"][0])
        data_point = {
            'ImageId': StringElement(file_name),
            'ImageUri': StringElement(f"https://ragaaimedia.s3.ap-south-1.amazonaws.com/1/StopSign_Part1.json/data_points/{pathlib.Path(file_name).stem}/{file_name}"),
            'TimeOfCapture': TimeStampElement(get_timestamp_x_hours_ago(hr)),
            'SourceLink': StringElement(file_name),
            'ModelAInfernences':ModelAInfernences,
            'ModelBInfernences':ModelBInfernences,
        }

        merged_dict = {**data_point, **attributes_dict}
        test_data_frame.append(merged_dict)
        hr+=1
        if main_index == 50:
            break
    return test_data_frame

#Convert JSON dataset to pandas Data Frame


pd_data_frame = pd.DataFrame(convert_json_to_data_frame("assets/video_models_A.json", "assets/video_models_A.json", "assets/video_models_A.json", "assets/video_models_A.json"))

print(data_frame_extractor(pd_data_frame).to_csv("TestingDataFrame_video.csv"))

# pd_data_frame = data_frame_extractor(pd.read_pickle("50_images_no_bbox_retail.pkl")).to_csv("50_images_classification.csv", index=False)
# print(pd_data_frame)

# pd_data_frame = pd.read_pickle("50_images_no_bbox_retail.pkl")

# schema = RagaSchema()
# schema.add("ImageId", PredictionSchemaElement(), pd_data_frame)
# schema.add("ImageUri", ImageUriSchemaElement(), pd_data_frame)
# schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_data_frame)
# schema.add("SourceLink", FeatureSchemaElement(), pd_data_frame)
# schema.add("Reflection", AttributeSchemaElement(), pd_data_frame)
# schema.add("Overlap", AttributeSchemaElement(), pd_data_frame)
# schema.add("CameraAngle", AttributeSchemaElement(), pd_data_frame)
# schema.add("ModelA", ImageClassificationSchemaElement(model="modelA"), pd_data_frame)
# schema.add("GT", ImageClassificationSchemaElement(model="GT"), pd_data_frame)
# schema.add("ModelAROI", RoiEmbeddingSchemaElement(model="modelA"), pd_data_frame)
# schema.add("GTROI", RoiEmbeddingSchemaElement(model="GT"), pd_data_frame)
# schema.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="imageModel"), pd_data_frame)


# # create test_session object of TestSession instance
# test_session = TestSession(project_name="testingProject",run_name= "run-18-aug-classification-v2")

# # # #create test_ds object of Dataset instance
# test_ds = Dataset(test_session=test_session, name="retail-dataset-50-images-aug-18-v1")

# # #load schema and pandas data frame
# test_ds.load(data=pd_data_frame, schema=schema)


from raga import *
import pandas as pd
import json
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def convert_json_to_data_frame(json_file_path_model_1, json_file_path_model_2, json_file_path_model_3, embeddings_file):
    test_data_frame = []
    with open(json_file_path_model_1, 'r') as json_file:
        # Load JSON data
        model_1 = json.load(json_file)
    with open(json_file_path_model_2, 'r') as json_file:
        # Load JSON data
        model_2 = json.load(json_file)
    
    with open(json_file_path_model_3, 'r') as json_file:
        # Load JSON data
        model_gt = json.load(json_file)
    
    with open(embeddings_file, 'r') as json_file:
        # Load JSON data
        embeddings_json = json.load(json_file)

    # Create a dictionary to store the inputs and corresponding data points
    inputs_dict_model_1 = {}
    hr = 1
    # Process model_1 data
    for item in model_1:
        inputs = item["inputs"]
        inputs_dict_model_1[tuple(inputs)] = item
    
    inputs_dict_model_3 = {}
    # Process model_3 data
    for item in model_gt:
        inputs = item["inputs"]
        inputs_dict_model_3[tuple(inputs)] = item

    
    inputs_dict_emb = {}
    # Process emb data
    for item in embeddings_json:
        inputs = item["inputs"]
        inputs_dict_emb[tuple(inputs)] = item

    # Process model_2 data
    for main_index, item in enumerate(model_2):
        inputs = item["inputs"]
        AnnotationsV1 = ImageDetectionObject()
        ROIVectorsM1 = ROIEmbedding()
        for index, detection in enumerate(item["outputs"][0]["detections"]):
            id = index+1
            AnnotationsV1.add(ObjectDetection(Id=id, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
            if detection['roi_embedding']:
                print('ROI3')
                ROIVectorsM1.add(id=id, embedding_values=[float(num_str) for num_str in detection['roi_embedding']])

            attributes_dict = {}
            attributes = item.get("attributes", {})
            for key, value in attributes.items():
                attributes_dict[key] = StringElement(value)
        merged_item = inputs_dict_model_1.get(tuple(inputs), {})
        AnnotationsV2 = ImageDetectionObject()
        ROIVectorsM2 = ROIEmbedding()
        for index, detection in enumerate(merged_item["outputs"][0]["detections"]):
            id = index+1
            AnnotationsV2.add(ObjectDetection(Id=id, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
            if detection['roi_embedding']:
                print('ROI2')
                ROIVectorsM2.add(id=id, embedding_values=[float(num_str) for num_str in detection['roi_embedding']])
        
        merged_item = inputs_dict_model_3.get(tuple(inputs), {})
        AnnotationsV3 = ImageDetectionObject()
        ROIVectorsM3 = ROIEmbedding()
        for index, detection in enumerate(merged_item["outputs"][0]["detections"]):
            id = index+1
            AnnotationsV3.add(ObjectDetection(Id=id, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
            if detection['roi_embedding']:
                print('ROI1')
                ROIVectorsM3.add(id=id, embedding_values=[float(num_str) for num_str in detection['roi_embedding']])
        
        merged_item = inputs_dict_emb.get(tuple(inputs), {})
        ImageVectorsM1 = ImageEmbedding()
        
        image_embeddings = merged_item.get("image_embedding", {})
        for value in image_embeddings:
            ImageVectorsM1.add(Embedding(value))

        file_name = os.path.basename(item["inputs"][0])
        data_point = {
            'ImageId': StringElement(file_name),
            'ImageUri': StringElement(f"https://nusceneraga.s3.ap-south-1.amazonaws.com/nuscene_rainy_images/{file_name}"),
            'TimeOfCapture': TimeStampElement(get_timestamp_x_hours_ago(hr)),
            'SourceLink': StringElement(file_name),
            'ModelA': AnnotationsV1,
            'ModelB': AnnotationsV2,
            'ModelG': AnnotationsV3,
            'ImageEmbedding': ImageVectorsM1,
        }

        merged_dict = {**data_point, **attributes_dict}
        test_data_frame.append(merged_dict)
        hr+=1
        if main_index == 50:
            break
    return test_data_frame

pd_data_frame = pd.DataFrame(convert_json_to_data_frame("ModelA_faster_rcnn_r50_fpn_nuscene_US_rainy_clear_engg_embedding.json", "ModelB_faster_rcnn_swin-t_nuscene_US_rainy_clear_engg_embedding.json", "GT_cascade_rcnn_swin-t_nuscene_US_rainy_clear_engg_embedding.json", "nuscene_US_rainy_clear_engg_embedding.json"))


schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement(), pd_data_frame)
schema.add("ImageUri", ImageUriSchemaElement(), pd_data_frame)
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_data_frame)
schema.add("SourceLink", FeatureSchemaElement(), pd_data_frame)
schema.add("location", AttributeSchemaElement(), pd_data_frame)
schema.add("vehicle_no", AttributeSchemaElement(), pd_data_frame)
schema.add("date_captured", AttributeSchemaElement(), pd_data_frame)
schema.add("channel", AttributeSchemaElement(), pd_data_frame)
schema.add("dataset_type", AttributeSchemaElement(), pd_data_frame)
schema.add("ModelA", InferenceSchemaElement(model="modelA"), pd_data_frame)
schema.add("ModelB", InferenceSchemaElement(model="modelB"), pd_data_frame)
schema.add("ModelG", InferenceSchemaElement(model="GT"), pd_data_frame)
schema.add("ImageEmbedding", ImageEmbeddingSchemaElement(model="imageModel"), pd_data_frame)

#create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject",run_name= "run-22-aug-v1")

# # #create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="dataset--aug-v3")

# #load schema and pandas data frame
test_ds.load(data=pd_data_frame, schema=schema)


import pathlib
from raga import *
import pandas as pd
import datetime

label_to_classname = {
    1: "no_data",
    2: "water",
    3: "trees",
    4: "grass",
    5: "flooded vegetation",
    6: "crops",
    7: "scrub",
    8: "built_area",
    9: "bare_ground",
    10: "snow_or_ice",
    11: "clouds",
}

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def image_url(x):
    return f"https://raga-testing-storage.s3.ap-south-1.amazonaws.com/1/satsure_rgb/{x}"

def mask_url(x):
    return f"https://raga-testing-storage.s3.ap-south-1.amazonaws.com/1/satsure_lulc/{x}"

def attached_label(id, loss):
    return {id:loss}

def merge_loss_dicts(loss_dicts):
    mistake_score = MistakeScore()
    for d in loss_dicts:
        for key, value in d.items():
            mistake_score.add(key=key, value=value)
    return mistake_score

def csv_parser(file_path):
    df = pd.read_csv(file_path)
    df['loss'] = df.apply(lambda row: attached_label(row['label'], row['loss']), axis=1)
    merged_df = df.groupby('filepath', as_index=False)['loss'].agg(merge_loss_dicts)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = merged_df["filepath"].apply(lambda x: StringElement(x))
    data_frame["ImageUri"] = merged_df["filepath"].apply(lambda x: StringElement(image_url(x)))
    data_frame["TimeOfCapture"] = merged_df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["SourceLink"] = merged_df['filepath'].apply(lambda x:StringElement(x))
    data_frame["Annotations"] = merged_df['filepath'].apply(lambda x:StringElement(mask_url(x)))
    data_frame["MistakeScores"] = merged_df['loss']
    data_frame["Reflection"] = data_frame.apply(lambda row: StringElement("Yes"), axis=1)
    data_frame["Overlap"] = data_frame.apply(lambda row: StringElement("No"), axis=1)
    data_frame["CameraAngle"] = data_frame.apply(lambda row: StringElement("Yes"), axis=1)
    return data_frame


data_frame = csv_parser("./assets/final_image_loss.csv")

#### Want to see in csv file uncomment line below ####
# data_frame_extractor(data_frame).to_csv("./assets/final_image_loss_full_s_df_new.csv", index=False)

schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement())
schema.add("SourceLink", FeatureSchemaElement())
schema.add("Annotations", TIFFSchemaElement(label_mapping=label_to_classname, schema="tiff"))
schema.add("MistakeScores", MistakeScoreSchemaElement(ref_col_name="Annotations"))
schema.add('Reflection', AttributeSchemaElement())
schema.add('Overlap', AttributeSchemaElement())
schema.add('CameraAngle', AttributeSchemaElement())



run_name = f"loader_labeling_quality_semantic_segmentation-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="dE4Zs3iT8LnYPP3KNNsq", secret_key="JzDgIoB1irU9V2vbHtvTSNLvRzAzjbJZab2cZLos", host="http://13.126.220.245:8080")

cred = DatasetCreds(arn="arn:aws:iam::527593518644:role/s3-role")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="satsure-full-dataset-v2", data=data_frame, schema=schema)

# #load schema and pandas data frame
test_ds.load()




import datetime
from raga.raga_schema import FMARules
from raga._tests import clustering, failure_mode_analysis
from raga.test_session import TestSession


run_name = f"run-failure-mode-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="nnXvot82D3idpraRtCjJ", secret_key="P2doycL4WBZXLNARIs4bESxttzF3MHSC5K15Jrs9", host="http://65.0.13.122:8080")

rules = FMARules()
rules.add(metric="Precision", conf_threshold=0.8, metric_threshold=0.5, iou_threshold=0.5, label="ALL")

cls_default = clustering(method="k-means", embedding_col="ImageVectorsM1", level="image", args= {"numOfClusters": 5})

edge_case_detection = failure_mode_analysis(test_session=test_session,
                                            dataset_name = "fma_image_object_detection_ds-v5",
                                            test_name = "FMA OD Test",
                                            model = "ModelB",
                                            gt = "GT",
                                            rules = rules,
                                            output_type="object_detection",
                                            type="embedding",
                                            clustering=cls_default
                                            )

test_session.add(edge_case_detection)

test_session.run()


import csv
import pathlib
from raga import *
import pandas as pd
import json
import datetime
import random

label_to_classname = {
    0: "no_data",
    1: "water",
    2: "trees",
    3: "grass",
    4: "flooded vegetation",
    5: "crops",
    6: "scrub",
    7: "built_area",
    8: "bare_ground",
    9: "snow_or_ice",
    10: "clouds",
}


def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def generate_random_float_list(length=18, min_value=0.0, max_value=500):
    if length <= 10:
        raise ValueError("Length should be greater than 10")
    random_list = [round(random.uniform(min_value, max_value),1) for _ in range(length)]
    return random_list

def parser():
    csv_file_path = "assets/final_image_loss.csv"

    csv_data = pd.read_csv(csv_file_path)
    df = []
    for index, row in csv_data.iterrows():
        if row["label"] in label_to_classname:
            Annotations = SemanticSegmentationObject()
            Loss = LossValue()
            filepath = row["filepath"]
            loss = row["loss"]
            label = label_to_classname[row["label"]]
            Annotations.add(SemanticSegmentation(Id=row["label"], ClassId=0, Segmentation=generate_random_float_list(),  ClassName=label, Format="xn,yn_normalised", Confidence=1))
            Loss.add(id=row["label"], values = loss)
            data_point = {
                'ImageId': StringElement(filepath),
                'ImageUri': StringElement(f"https://raga-test-bucket.s3.ap-south-1.amazonaws.com/1/satsure/data_points/{pathlib.Path(filepath).stem}/images/{filepath}"),
                'TimeOfCapture': TimeStampElement(get_timestamp_x_hours_ago(index)),
                'SourceLink': StringElement(filepath),
                'Annotations': Annotations,
                'LossValue':Loss
            }
            df.append(data_point)
    return pd.DataFrame(df)


# print(data_frame_extractor(parser()).to_csv("satsure.csv"))

pd_data_frame = parser()
schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement(), pd_data_frame)
schema.add("ImageUri", ImageUriSchemaElement(), pd_data_frame)
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_data_frame)
schema.add("SourceLink", FeatureSchemaElement(), pd_data_frame)
schema.add("Annotations", SemanticSegmentationSchemaElement(), pd_data_frame)
schema.add("LossValue", LossValueSchemaElement(), pd_data_frame)

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject",run_name= "labelling-dataset-run-23-aug-v1")

# # #create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="labelling-dataset-23-aug-v1")

# #load schema and pandas data frame
test_ds.load(data=pd_data_frame, schema=schema)


from raga import *
import datetime


run_name = f"failure_mode_analysis_semantic_segmentation-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name= run_name, access_key="nnXvot82D3idpraRtCjJ", secret_key="P2doycL4WBZXLNARIs4bESxttzF3MHSC5K15Jrs9", host="http://65.0.13.122:8080")

rules = FMARules()

rules.add(metric = "F1Score",  metric_threshold = 0.5, label = "ALL", type="label", background_label="unlabelled", include_background=False)

rules.add(metric = "wIoU", metric_threshold = 0.5, weights={"car": 2, "unlabelled": 1}, type="label")



cls_default = clustering(method="k-means", embedding_col="ImageVectorsM1", level="image", args= {"numOfClusters": 5})

edge_case_detection = failure_mode_analysis(test_session=test_session,
                                            dataset_name = "satellite_image_dataset_v3",
                                            test_name = "Test",
                                            model = "ModelA",
                                            gt = "ModelB",
                                            rules = rules,
                                            output_type="semantic_segmentation",
                                            type="metadata",
                                            aggregation_level=['CameraAngle'])

test_session.add(edge_case_detection)

test_session.run()


import ast
from raga import *
import pandas as pd
import json
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def img_url(x):
    return StringElement(f"https://raga-testing-storage.s3.ap-south-1.amazonaws.com/1/signzy/{x.replace('./', '')}")

def model_a_inference(row):
    classification = ImageClassificationElement()
    try:
        confidence = ast.literal_eval(row['ModelA Inference'])
        classification.add("live", confidence.get('live'))
    except Exception as exc:
        classification.add("live", 0)
    return classification


def model_b_inference(row):
    classification = ImageClassificationElement()
    try:
        confidence = ast.literal_eval(row['ModelB Inference'])
        classification.add("live", confidence.get('live'))
    except Exception as exc:
        classification.add("live", 0)
    return classification


def model_gt_inference(row):
    classification = ImageClassificationElement()
    try:
        confidence = ast.literal_eval(row['Ground Truth'])
        classification.add("live", confidence.get('live'))
    except Exception as exc:
        classification.add("live", 0)
    return classification

def image_vectors_m1(row):
    ImageVectorsM1 = ImageEmbedding()
    for embedding in json.loads(row['ImageVectorsM1']):
        ImageVectorsM1.add(Embedding(embedding))
    return ImageVectorsM1


def csv_parser(csv_file):
    df = pd.read_csv(csv_file)
    data_frame = pd.DataFrame()
    data_frame["ImageId"] = df["ImageId"].apply(lambda x: StringElement(x))
    data_frame["ImageUri"] = df["SourceLink"].apply(lambda x: img_url(x))
    data_frame["TimeOfCapture"] = df.apply(lambda row: TimeStampElement(get_timestamp_x_hours_ago(row.name)), axis=1)
    data_frame["Reflection"] = df.apply(lambda row: StringElement("Yes"), axis=1)
    data_frame["Overlap"] = df.apply(lambda row: StringElement("No"), axis=1)
    data_frame["CameraAngle"] = df.apply(lambda row: StringElement("Yes"), axis=1)
    data_frame["SourceLink"] = df["SourceLink"].apply(lambda x: StringElement(x))
    data_frame["ModelA Inference"] = df.apply(model_a_inference, axis=1)
    data_frame["ModelB Inference"] = df.apply(model_b_inference, axis=1)
    data_frame["Ground Truth"] = df.apply(model_gt_inference, axis=1)
    data_frame["ImageVectorsM1"] =  df.apply(image_vectors_m1, axis=1)
    return data_frame


pd_data_frame = csv_parser("./assets/signzy_df.csv")

#### Want to see in csv file uncomment line below ####
# data_frame_extractor(pd_data_frame).to_csv("signzy_df_csv.csv", index=False)


schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement())
schema.add("ImageUri", ImageUriSchemaElement())
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement())
schema.add("SourceLink", FeatureSchemaElement())
schema.add("ModelA Inference", ImageClassificationSchemaElement(model="modelA"))
# schema.add("ModelB Inference", ImageClassificationSchemaElement(model="modelB"))
# schema.add("Ground Truth", ImageClassificationSchemaElement(model="GT"))
schema.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="imageModel"))

run_name = f"loader_failure_mode_analysis_image_classification-{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}"

# create test_session object of TestSession instance
test_session = TestSession(project_name="LivenessDetection", run_name= run_name, access_key="u4amNsYmYE0hQWCPPFQ8", secret_key="FUDZ3NxORAqyCCmY7eezJEU9gxX0uPM232V1g75W", host="http://13.126.220.245:8080")

cred = DatasetCreds(arn="arn:aws:iam::527593518644:role/s3-role")

#create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="live-dataset-a", data=pd_data_frame, schema=schema, creds=cred)

#load schema and pandas data frame
test_ds.load()


from raga import *
import pandas as pd
import json
import datetime

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def convert_json_to_data_frame(json_file_path_model_1, json_file_path_model_2, json_file_path_model_3):
    test_data_frame = []
    with open(json_file_path_model_1, 'r') as json_file:
        # Load JSON data
        model_1 = json.load(json_file)
    with open(json_file_path_model_2, 'r') as json_file:
        # Load JSON data
        model_2 = json.load(json_file)
    
    with open(json_file_path_model_3, 'r') as json_file:
        # Load JSON data
        model_gt = json.load(json_file)

    # Create a dictionary to store the inputs and corresponding data points
    inputs_dict = {}
    hr = 1
    # Process model_1 data
    for item in model_1:
        inputs = item["inputs"]
        inputs_dict[tuple(inputs)] = item
    
    # Process model_2 data
    for item in model_2:
        inputs = item["inputs"]
        AnnotationsV1 = ImageDetectionObject()
        ROIVectorsM1 = ROIEmbedding()
        ImageVectorsM1 = ImageEmbedding()
        for detection in item["outputs"][0]["detections"]:
            AnnotationsV1.add(ObjectDetection(Id=0, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
            for roi_emb in detection['roi_embedding']:
                ROIVectorsM1.add(Embedding(roi_emb))
            attributes_dict = {}
            attributes = item.get("attributes", {})
            for key, value in attributes.items():
                attributes_dict[key] = StringElement(value)
            image_embeddings = item.get("image_embedding", {})
            for value in image_embeddings:
                ImageVectorsM1.add(Embedding(value))

        merged_item = inputs_dict.get(tuple(inputs), {})
        AnnotationsV2 = ImageDetectionObject()
        ROIVectorsM2 = ROIEmbedding()
        ImageVectorsM2 = ImageEmbedding()
        for detection in merged_item["outputs"][0]["detections"]:
            AnnotationsV2.add(ObjectDetection(Id=0, ClassId=0, ClassName=detection['class'], Confidence=detection['confidence'], BBox= detection['bbox'], Format="xywh_normalized"))
            for roi_emb in detection['roi_embedding']:
                ROIVectorsM2.add(Embedding(roi_emb))
        
        image_embeddings = merged_item.get("image_embedding", {})
        for value in image_embeddings:
            ImageVectorsM2.add(Embedding(value))

        data_point = {
            'ImageId': StringElement(item["inputs"][0]),
            'TimeOfCapture': TimeStampElement(get_timestamp_x_hours_ago(hr)),
            'SourceLink': StringElement(item["inputs"][0]),
            'AnnotationsV1': AnnotationsV1,
            'ROIVectorsM1': ROIVectorsM1,
            'ImageVectorsM1': ImageVectorsM1,
            'AnnotationsV2': AnnotationsV2,
            'ROIVectorsM2': ROIVectorsM2,
            'ImageVectorsM2': ImageVectorsM2,
        }

        merged_dict = {**data_point, **attributes_dict}
        test_data_frame.append(merged_dict)
        hr+=1

    return test_data_frame



#Convert JSON dataset to pandas Data Frame
pd_data_frame = pd.DataFrame(convert_json_to_data_frame("test-dataset-modelA.json", "test-dataset-modelB.json", "test-dataset-modelG.json"))

pd_data_frame.to_pickle("TestingDataFrame.pkl")


import os
import configparser
from raga import RAGA_CONFIG_FILE
import sys
import platform

DEFAULT_CONFIG_VALUES = {
    "api_host": "https://example.com",
    "raga_access_key_id": "your-access-key",
    "raga_secret_access_key": "your-secret-key"
}

def get_config_file_path(path):
    if platform.system() == 'Linux':
        # Path for Linux machine
        return os.path.expanduser(os.path.join("~", path))
    elif platform.system() == 'Darwin':
        # Path for macOS machine (if required, add a different path for macOS)
        return os.path.expanduser(os.path.join("~", path))
    elif platform.system() == 'Windows':
        # Path for Windows machine (if required, add a different path for Windows)
        return os.path.expanduser(os.path.join("~", path))
    else:
        # Default path for other platforms (such as Google Colab)
        return '/content/MyDrive.raga/config'

def get_machine_platform():
        system = platform.system().lower()

        if system == "darwin":
            mac_version = "11_0" if platform.release() >= "20.0.0" else "10_9"
            arch = "arm64" if platform.machine() == "arm64" else "x86_64"
            platform_name = f"macosx_{mac_version}_{arch}"
        elif system == "linux":
            platform_name = (
                "linux_x86_64"  # if platform.architecture()[0] == '64bit' else 'linux_i686'
            )
        elif system == 'windows':
            platform_name = 'win_amd64' if platform.architecture()[0] == '64bit' else 'win32'
        else:
            raise RuntimeError(f"Unsupported platform: {system}")
        return platform_name
        
def get_python_version():
    version = sys.version.split()[0]
    major, minor, _ = map(int, version.split("."))
    return str(f"cp{major}{minor}")


def format_python_versions(version_str):
    versions = version_str.split(',')
    formatted_versions = []

    for version in versions:
        parts = version.split('.')
        major = int(parts[0])
        minor = int(parts[1])

        version = f"{major}.{minor}"
        formatted_versions.append(float(version))

    return formatted_versions


def read_raga_config():
    config_file_path = get_config_file_path(RAGA_CONFIG_FILE)

    if not os.path.isfile(config_file_path):
        create_default_config(config_file_path)
        print(f"A default config file has been created. Please update the credentials in the config file. You can update using this command `sudo vim {config_file_path}`")
        sys.exit(0)

    config = configparser.ConfigParser()
    try:
        config.read(config_file_path)
    except configparser.Error as e:
        raise ValueError(f"Invalid config file format: {str(e)}")

    validate_default_section(config)

    config_data = {}
    for section_name in config.sections():
        config_data[section_name] = dict(config.items(section_name))

    return config_data

def create_default_config(config_file_path):
    config = configparser.ConfigParser()
    config.add_section("default")

    for option, value in DEFAULT_CONFIG_VALUES.items():
        config.set("default", option, value)

    os.makedirs(os.path.dirname(config_file_path), exist_ok=True)

    with open(config_file_path, "w") as config_file:
        config.write(config_file)

def validate_default_section(config):
    default_section = config["default"]

    for option, default_value in DEFAULT_CONFIG_VALUES.items():
        if option not in default_section or default_section[option] == default_value:
            raise ValueError(f"Please update the value of '{option}' in the [default] section of the config file.")

def get_config_value(config_data, section, option):
    if section in config_data:
        section_data = config_data[section]
        if option in section_data:
            return section_data[option]
        else:
            raise KeyError(f"Option '{option}' not found in section '{section}'.")
    else:
        raise KeyError(f"Section '{section}' not found in config data.")



import ast
import csv
import pathlib
import re
from raga import *
import pandas as pd
import json
import datetime
import random

def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def convert_json_to_data_frame(json_file_path_model_1, json_file_path_model_2, json_file_path_model_3, embeddings_file=None):
    test_data_frame = []
    with open(json_file_path_model_1, 'r') as json_file:
        # Load JSON data
        model_1 = json.load(json_file)
    with open(json_file_path_model_2, 'r') as json_file:
        # Load JSON data
        model_2 = json.load(json_file)
    
    with open(json_file_path_model_3, 'r') as json_file:
        # Load JSON data
        model_gt = json.load(json_file)
    if embeddings_file:
        with open(embeddings_file, 'r') as json_file:
            # Load JSON data
            embeddings_json = json.load(json_file)

    # Create a dictionary to store the inputs and corresponding data points
    inputs_dict_model_1 = {}
    hr = 1
    # Process model_1 data
    for item in model_1:
        inputs = item["inputs"]
        inputs_dict_model_1[tuple(inputs)] = item
    
    inputs_dict_model_3 = {}
    # Process model_3 data
    for item in model_gt:
        inputs = item["inputs"]
        inputs_dict_model_3[tuple(inputs)] = item

    
    inputs_dict_emb = {}
    # Process emb data
    for item in embeddings_json:
        inputs = item["inputs"]
        inputs_dict_emb[tuple(inputs)] = item

    # Process model_2 data
    for main_index, item in enumerate(model_2):
        ModelAImageClassification = ImageClassificationElement()
        ModelBImageClassification = ImageClassificationElement()
        ModelGImageClassification = ImageClassificationElement()
        inputs = item["inputs"]
        AnnotationsV1 = ImageDetectionObject()
        ROIVectorsM1 = ROIEmbedding()
        for index, detection in enumerate(item["outputs"][0]["detections"]):
            id = index+1
            if detection['roi_embedding']:
                print('ROI3')
                ROIVectorsM1.add(id=id, embedding_values=[float(num_str) for num_str in detection['roi_embedding']])

            attributes_dict = {}
            attributes = item.get("attributes", {})
            for key, value in attributes.items():
                attributes_dict[key] = StringElement(value)
        merged_item = inputs_dict_model_1.get(tuple(inputs), {})
        AnnotationsV2 = ImageDetectionObject()
        ROIVectorsM2 = ROIEmbedding()
        for index, detection in enumerate(merged_item["outputs"][0]["detections"]):
            id = index+1
            AnnotationsV2.add(ObjectDetection(Id=id, ClassId=0, ClassName=detection['class'], Confidence=round(random.uniform(0, 1), 1), BBox= [], Format="xywh_normalized"))
            if detection['roi_embedding']:
                print('ROI2')
                ROIVectorsM2.add(id=id, embedding_values=[float(num_str) for num_str in detection['roi_embedding']])
        
        merged_item = inputs_dict_model_3.get(tuple(inputs), {})
        AnnotationsV3 = ImageDetectionObject()
        ROIVectorsM3 = ROIEmbedding()
        for index, detection in enumerate(merged_item["outputs"][0]["detections"]):
            id = index+1
            ModelAImageClassification.add(key=detection['class'], value=round(random.uniform(0, 1), 1))
            ModelGImageClassification.add(key=detection['class'], value=random.randint(0, 1))
            if detection['roi_embedding']:
                print('ROI1')
                ROIVectorsM3.add(id=id, embedding_values=[float(num_str) for num_str in detection['roi_embedding']])
        
        merged_item = inputs_dict_emb.get(tuple(inputs), {})
        ImageVectorsM1 = ImageEmbedding()
        
        image_embeddings = merged_item.get("image_embedding", {})
        for value in image_embeddings:
            ImageVectorsM1.add(Embedding(value))

        file_name = os.path.basename(item["inputs"][0])
        data_point = {
            'ImageId': StringElement(file_name),
            'ImageUri': StringElement(f"https://ragaaimedia.s3.ap-south-1.amazonaws.com/54/self_serve-retail_checkout/data_points/{pathlib.Path(file_name).stem}/{file_name}"),
            'TimeOfCapture': TimeStampElement(get_timestamp_x_hours_ago(hr)),
            'SourceLink': StringElement(file_name),
            'ModelA': ModelAImageClassification,
            'GT': ModelGImageClassification,
            'ModelAROI': ROIVectorsM1,
            'GTROI': ROIVectorsM3,
            'ImageVectorsM1':ImageVectorsM1
        }

        merged_dict = {**data_point, **attributes_dict}
        test_data_frame.append(merged_dict)
        hr+=1
        if main_index == 50:
            break
    return test_data_frame

#Convert JSON dataset to pandas Data Frame

# pd_data_frame = pd.DataFrame(convert_json_to_data_frame("assets/mb.json", "assets/ma.json", "assets/gt.json", "assets/ma.json"))

# data_frame_extractor(pd_data_frame).to_pickle("50_images_no_bbox_retail.pkl")

# pd_data_frame = data_frame_extractor(pd.read_pickle("assets/50_images_no_bbox_retail.pkl")).to_csv("assets/50_images_classification.csv", index=False)
# print(pd_data_frame)


# pd_data_frame = pd.read_pickle("assets/50_images_no_bbox_retail.pkl")
pd_data_frame = pd.read_csv("assets/signzy_df.csv")
dr = []
for index, row in pd_data_frame.iterrows():
    df = {}
    try:
        for index_r, column_name in enumerate(pd_data_frame.columns):
            if column_name == "ModelA Inference":
                pass
                # element = row[column_name]
                # conf = ast.literal_eval(element)
                # del conf['spoof']
                # df[column_name] = {'confidence':conf}
            elif column_name == "ModelB Inference":
                element = row[column_name]
                conf = ast.literal_eval(element)
                del conf['spoof']
                df[column_name] = {'confidence':conf}

            elif column_name == "Ground Truth":
                element = row[column_name]
                conf = ast.literal_eval(element)
                del conf['spoof']
                df[column_name] = {'confidence':conf}
                
            elif column_name == "ImageVectorsM1":
                element = row[column_name]
                element = json.loads(element)
                element_value = {'embeddings':element}
                df[column_name] = element_value
            # elif column_name == "Weather":
            #     element = row[column_name]
            #     df[column_name] = None
            # elif column_name == "TimeofDay":
            #     element = row[column_name]
            #     df[column_name] = ""
            elif column_name == "TimeOfCapture":
                element = row[column_name]
                df[column_name] = get_timestamp_x_hours_ago(index_r)
            elif column_name == "ImageId":
                element = row[column_name]
                df[column_name] = element
                df["ImageUri"] = f"https://raga-test-bucket.s3.ap-south-1.amazonaws.com/spoof/{element}"
            else:
                element = row[column_name]
                element_value = element
                df[column_name] = element_value
    except Exception as e:
            continue
    dr.append(df)
pd_data_frame = pd.DataFrame(dr)


pd_data_frame.to_csv("signzy_df_1.csv", index=False)

pd_data_frame = pd.read_csv("signzy_df_1.csv")


schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement(), pd_data_frame)
schema.add("ImageUri", ImageUriSchemaElement(), pd_data_frame)
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), pd_data_frame)
schema.add("SourceLink", FeatureSchemaElement(), pd_data_frame)
# schema.add("Weather", AttributeSchemaElement(), pd_data_frame)
# schema.add("TimeofDay", AttributeSchemaElement(), pd_data_frame)
# schema.add("ModelA Inference", ImageClassificationSchemaElement(model="modelA"), pd_data_frame)
schema.add("ModelB Inference", ImageClassificationSchemaElement(model="modelB"), pd_data_frame)
schema.add("Ground Truth", ImageClassificationSchemaElement(model="GT"), pd_data_frame)
schema.add("ImageVectorsM1", ImageEmbeddingSchemaElement(model="imageModel"), pd_data_frame)


# create test_session object of TestSession instance
test_session = TestSession(project_name="LivenessDetection", run_name= "run-29-aug-classification-v9", access_key="xOVWFaP2fUB53jJkeflD", secret_key="RdhCitkCBLeq0YhA0VSlrSEeb8nbaVyKALF8dgBB", host="http://13.200.65.88:8080")

# test_session = TestSession(project_name="testingProject", run_name= "run-29-aug-classification-v9", access_key="nnXvot82D3idpraRtCjJ", secret_key="P2doycL4WBZXLNARIs4bESxttzF3MHSC5K15Jrs9", host="http://65.0.13.122:8080")

# # #create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name="live-dataset")

# #load schema and pandas data frame
test_ds.load(data=pd_data_frame, schema=schema)


import csv
import pathlib
from raga import *
import pandas as pd
import json
import datetime
import random

label_to_classname = {
    0: "no_data",
    1: "water",
    2: "trees",
    3: "grass",
    4: "flooded vegetation",
    5: "crops",
    6: "scrub",
    7: "built_area",
    8: "bare_ground",
    9: "snow_or_ice",
    10: "clouds",
}


def get_timestamp_x_hours_ago(hours):
    current_time = datetime.datetime.now()
    delta = datetime.timedelta(days=90, hours=hours)
    past_time = current_time - delta
    timestamp = int(past_time.timestamp())
    return timestamp

def generate_random_float_list(length=18, min_value=0.0, max_value=500):
    if length <= 10:
        raise ValueError("Length should be greater than 10")
    random_list = [round(random.uniform(min_value, max_value),1) for _ in range(length)]
    return random_list


def parseLossFile(csv_file_path):
    df = pd.read_csv(csv_file_path)
    f2l = {}
    file_paths = []
    for index, row in df.iterrows():

        if row['label'] not in label_to_classname:
            continue

        file_path = row['filepath']
        if file_path not in f2l:
            file_paths.append(file_path)
            f2l[file_path] = []
        f2l[file_path].append([row['label'], row['loss']])

    return file_paths, f2l


def generate_data_frame(file_paths, f2l):
    data = []
    for filepath in file_paths:

        annotations = SemanticSegmentationObject()
        loss_values = LossValue()

        for i, item in enumerate(f2l[filepath]):
            if item[0] not in label_to_classname:
                continue
            label = label_to_classname[item[0]]
            segmentation = generate_random_float_list()
            annotations.add(SemanticSegmentation(Id=i, ClassId=item[0], ClassName=label, Segmentation=segmentation, Confidence=1, Format="xn,yn_normalised"))
            loss_values.add(id=i, values=item[1])

        image_url = f"https://raga-test-bucket.s3.ap-south-1.amazonaws.com/1/satsure/data_points/{pathlib.Path(filepath).stem}/images/{filepath}"
        timestamp = get_timestamp_x_hours_ago(i)
        data_point = {
            'ImageId': StringElement(filepath),
            'ImageUri': StringElement(image_url),
            'TimeOfCapture': TimeStampElement(timestamp),
            'SourceLink': StringElement(filepath),
            'Annotations': annotations,
            'LossValue': loss_values,
            'Reflection': StringElement('Yes'),
            'Overlap': StringElement('Yes'),
            'CameraAngle': StringElement('Top')
        }
        data.append(data_point)

    return pd.DataFrame(data)


csv_file_path = "./assets/final_image_loss_test.csv"
dataset_name = "satsure-dataset"
experiment_name = "satsure-aug-30-v1"

file_paths, f2l = parseLossFile(csv_file_path)
df = generate_data_frame(file_paths, f2l)


schema = RagaSchema()
schema.add("ImageId", PredictionSchemaElement(), df)
schema.add("ImageUri", ImageUriSchemaElement(), df)
schema.add("TimeOfCapture", TimeOfCaptureSchemaElement(), df)
schema.add("SourceLink", FeatureSchemaElement(), df)
schema.add("Annotations", SemanticSegmentationSchemaElement(), df)
schema.add("LossValue", LossValueSchemaElement(), df)
schema.add('Reflection', AttributeSchemaElement(), df)
schema.add('Overlap', AttributeSchemaElement(), df)
schema.add('CameraAngle', AttributeSchemaElement(), df)

# create test_session object of TestSession instance
test_session = TestSession(project_name="testingProject", run_name=experiment_name)

# # #create test_ds object of Dataset instance
test_ds = Dataset(test_session=test_session, name=dataset_name)

# #load schema and pandas data frame
test_ds.load(data=df, schema=schema)


from raga import *


test_session = TestSession(project_name="LivenessDetection",
                           run_name="clustering-25-aug-v1")

rules = FMARules()
rules.add(metric="Precision", conf_threshold=0.8, metric_threshold=0.5, label="ALL")

cls_default = clustering(method="k-means", embedding_col="ImageVectorsM1", level="image", args= {"numOfClusters": 5})

edge_case_detection = failure_mode_analysis(test_session=test_session,
                                            dataset_name = "spoof-dataset",
                                            test_name = "Test",
                                            model = "modelB",
                                            gt = "GT",
                                            clustering = cls_default,
                                            rules = rules,
                                            output_type="multi-label")

test_session.add(edge_case_detection)

test_session.run()